import datetime

from cloud_governance.policy.policy_operations.aws.zombie_non_cluster.run_zombie_non_cluster_policies import NonClusterZombiePolicy


class EC2Run(NonClusterZombiePolicy):

    def __init__(self):
        super(EC2Run, self).__init__()
        self.__es_index = 'cloud-governance-ec2-instance-types'

    def run(self):
        """
        This method list all in-use ebs volumes
        @return:
        """
        running_instances = []
        instances = self._ec2_operations.get_instances()
        instance_types = {}
        for instance in instances:
            for resource in instance['Instances']:
                if resource.get('State').get('Name') == 'running':
                    running_instances.append(resource)
                instance_type = resource.get('InstanceType')
                instance_types[instance_type] = instance_types.get(instance_type, 0)+1
        es_instance_types_data = []
        for key, value in instance_types.items():
            es_instance_types_data.append({
                'instance_type': key,
                'instance_count': value,
                'timestamp': datetime.datetime.utcnow(),
                'region': self._region,
                'account': self._account.upper().replace('OPENSHIFT-', ''),
                'index_id': f'{key}-{self._account.lower()}-{self._region}-{str(datetime.datetime.utcnow().date())}'
            })
        self._es_upload.es_upload_data(items=es_instance_types_data, es_index=self.__es_index, set_index='index_id')
        return self._organise_instance_data(running_instances)



import datetime
import operator

from cloud_governance.common.clouds.aws.cloudtrail.cloudtrail_operations import CloudTrailOperations
from cloud_governance.common.logger.init_logger import logger
from cloud_governance.policy.policy_operations.aws.zombie_non_cluster.run_zombie_non_cluster_policies import NonClusterZombiePolicy
from operator import ge


class EC2Stop(NonClusterZombiePolicy):
    """
    This class delete the stopped ec2 instance
    Send mail to the user if the ec2 instance is stopped > 20, 25th days
    Delete the instance when it stopped 30 days
    """

    FIRST_MAIL_NOTIFICATION_INSTANCE_DAYS = 20
    SECOND_MAIL_NOTIFICATION_INSTANCE_DAYS = 25
    DELETE_INSTANCE_DAYS = 30
    DAYS_TO_NOTIFY_ADMINS = 26

    def __init__(self):
        super().__init__()
        self._cloudtrail = CloudTrailOperations(region_name=self._region)

    def run(self):
        """
        This method list all stopped instances for more than 30 days and terminate if dry_run no
        @return:
        """
        return self.__fetch_stop_instance(sign=ge, instance_days=self.FIRST_MAIL_NOTIFICATION_INSTANCE_DAYS, delete_instance_days=self.DELETE_INSTANCE_DAYS)

    def __fetch_stop_instance(self, instance_days: int, delete_instance_days: int, sign: operator = ge):
        """
        This method list all stopped instances for more than 30 days and terminate if dry_run no
        @return:
        """
        instances = self._ec2_client.describe_instances(Filters=[{'Name': 'instance-state-name', 'Values': ['stopped']}])['Reservations']
        stopped_instances = []
        stopped_instance_tags = {}
        ec2_types = {}
        block_device_mappings = {}
        stopped_time = ''
        days = 0
        for instance in instances:
            for resource in instance['Instances']:
                if self._get_policy_value(tags=resource.get('Tags', [])) not in ('NOTDELETE', 'SKIP'):
                    instance_id = resource.get('InstanceId')
                    stopped_time = self._cloudtrail.get_stop_time(resource_id=instance_id, event_name='StopInstances')
                    if not stopped_time:
                        stopped_time = datetime.datetime.now()
                    days = self._calculate_days(create_date=stopped_time)
                    user = self._get_tag_name_from_tags(tags=resource.get('Tags'), tag_name='User')
                    stop_cost = self.get_ebs_cost(resource=resource.get('BlockDeviceMappings'), resource_type='ec2', resource_hours=(self.DAILY_HOURS * days))
                    if days in (instance_days, self.SECOND_MAIL_NOTIFICATION_INSTANCE_DAYS):
                        if days == self.SECOND_MAIL_NOTIFICATION_INSTANCE_DAYS:
                            delta_cost = self.get_ebs_cost(resource=resource.get('BlockDeviceMappings'), resource_type='ec2', resource_hours=(self.DAILY_HOURS * (days-self.FIRST_MAIL_NOTIFICATION_INSTANCE_DAYS)))
                        else:
                            delta_cost = stop_cost
                        if user:
                            self.__trigger_mail(tags=resource.get('Tags'), stopped_time=stopped_time, resource_id=instance_id, days=days, ec2_type=resource.get("InstanceType"), instance_id=instance_id, message_type='notification', stop_cost=stop_cost, delta_cost=delta_cost)
                        else:
                            logger.info('User is missing')
                    if days == self.DAYS_TO_NOTIFY_ADMINS:
                        delta_charge = self.get_ebs_cost(resource=resource.get('BlockDeviceMappings'), resource_type='ec2', resource_hours=(self.DAILY_HOURS * (self.DAYS_TO_NOTIFY_ADMINS-self.SECOND_MAIL_NOTIFICATION_INSTANCE_DAYS)))
                        self.__trigger_mail(tags=resource.get('Tags'), stopped_time=stopped_time, resource_id=instance_id,
                                            days=days, ec2_type=resource.get("InstanceType"), instance_id=instance_id, admins=self._admins,
                                            message_type='notify-admin', stop_cost=stop_cost, delta_charge=delta_charge)
                    if sign(days, instance_days):
                        if days >= delete_instance_days:
                            stopped_instance_tags[instance_id] = resource.get('Tags')
                            ec2_types[instance_id] = resource.get('InstanceType')
                            block_device_mappings[instance_id] = resource.get('BlockDeviceMappings')
                        stopped_instances.append({
                            'policy': self._policy,
                            'ResourceId': resource.get('InstanceId'),
                            'StoppedDays': days,
                            'StoppedDate': str(resource.get('UsageOperationUpdateTime')),
                            'Name': self._get_tag_name_from_tags(tags=resource.get('Tags'), tag_name='Name'),
                            'User': self._get_tag_name_from_tags(tags=resource.get('Tags'), tag_name='User'),
                            'LaunchTime': str(resource.get('LaunchTime')),
                            'Policy': self._get_tag_name_from_tags(tags=resource.get('Tags'), tag_name='Policy')})
        if self._dry_run == "no":
            for instance_id, tags in stopped_instance_tags.items():
                if self._get_policy_value(tags=tags) not in ('NOTDELETE', 'SKIP'):
                    tags.append({'Key': 'Policy', 'Value': 'backup'})
                    tag_specifications = [{'ResourceType': 'image', 'Tags': tags}]
                    if sign == ge:
                        tag_specifications.append({'ResourceType': 'snapshot', 'Tags': tags})
                    try:
                        ami_id = self._ec2_client.create_image(InstanceId=instance_id, Name=self._get_tag_name_from_tags(tags=tags), TagSpecifications=tag_specifications)['ImageId']
                        delta_charge = self.get_ebs_cost(resource=block_device_mappings[instance_id], resource_type='ec2', resource_hours=(self.DAILY_HOURS * (self.DELETE_INSTANCE_DAYS-self.DAYS_TO_NOTIFY_ADMINS)))
                        stop_cost = self.get_ebs_cost(resource=block_device_mappings[instance_id], resource_type='ec2', resource_hours=(self.DAILY_HOURS * days))
                        self.__trigger_mail(tags=tags, stopped_time=stopped_time, days=days, resource_id=instance_id, image_id=ami_id, ec2_type=ec2_types[instance_id], instance_id=instance_id, message_type='delete', stop_cost=stop_cost, delta_charge=delta_charge)
                        self._ec2_client.terminate_instances(InstanceIds=[instance_id])
                        logger.info(f'Deleted the instance: {instance_id}')
                    except Exception as err:
                        logger.info(err)
        return stopped_instances

    def __trigger_mail(self, tags: list, stopped_time: str, resource_id: str, days: int, image_id: str = '', ec2_type: str = '', instance_id: str = '', **kwargs):
        """
        This method send triggering mail
        @param tags:
        @param stopped_time:
        @param resource_id:
        @param days:
        @return:
        """
        try:
            special_user_mails = self._literal_eval(self._special_user_mails)
            user, instance_name = self._get_tag_name_from_tags(tags=tags, tag_name='User'), self._get_tag_name_from_tags(tags=tags, tag_name='Name')
            to = user if user not in special_user_mails else special_user_mails[user]
            ldap_data = self._ldap.get_user_details(user_name=to)
            cc = []
            subject, body = self._mail_description.ec2_stop(name=ldap_data.get('displayName'), days=days, image_id=image_id, delete_instance_days=self.DELETE_INSTANCE_DAYS, instance_name=instance_name, resource_id=resource_id, stopped_time=stopped_time, ec2_type=ec2_type, extra_purse=kwargs.get('stop_cost'))
            if not kwargs.get('admins'):
                kwargs['admins'] = to
                cc = [self._account_admin, f'{ldap_data.get("managerId")}@redhat.com']
            else:
                kwargs['admins'].append(f'{ldap_data.get("managerId")}@redhat.com')
            self._mail.send_email_postfix(to=kwargs.get('admins'), content=body, subject=subject, cc=cc, resource_id=instance_id, message_type=kwargs.get('message_type'), extra_purse=kwargs.get('delta_cost', 0))
        except Exception as err:
            logger.info(err)



#!/usr/bin/env python3
import asyncio
import logging

import aiohttp
import urllib3
from aiohttp import BasicAuth

urllib3.disable_warnings()

logger = logging.getLogger(__name__)


class JiraException(Exception):
    pass


class Jira(object):
    def __init__(
        self,
        url,
        username,
        ticket_queue,
        password=None,
        token=None,
        loop=None,
    ):
        try:
            logger.debug(":Initializing Jira object:")
            self.url = url
            self.username = username
            self.ticket_queue = ticket_queue
            self.password = password
            if not loop:
                self.loop = asyncio.new_event_loop()
                self.new_loop = True
            else:
                self.loop = loop
                self.new_loop = False
            self.token = token
            if not self.token:
                if self.password:
                    payload = BasicAuth(self.username, self.password)
                else:
                    logger.error(
                        "Basic Authentication expected as no token was found but password is missing"
                    )
                    raise JiraException
            else:
                payload = "Bearer: %s" % self.token
            self.headers = {"Authorization": payload}
        except:
            pass

    def __exit__(self):
        if self.new_loop:
            self.loop.close()

    async def get_request(self, endpoint):
        logger.debug("GET: %s" % endpoint)
        try:
            async with aiohttp.ClientSession(
                headers=self.headers,
                loop=self.loop,
            ) as session:
                async with session.get(
                    self.url + endpoint,
                    verify_ssl=False,
                ) as response:
                    result = await response.json(content_type="application/json")
        except Exception as ex:
            logger.debug(ex)
            logger.error("There was something wrong with your request.")
            return None
        if response.status == 404:
            logger.error("Resource not found: %s" % self.url + endpoint)
            return None
        return result

    async def post_request(self, endpoint, payload):
        logger.debug("POST: {%s:%s}" % (endpoint, payload))
        try:
            async with aiohttp.ClientSession(
                headers=self.headers, loop=self.loop
            ) as session:
                async with session.post(
                    self.url + endpoint,
                    json=payload,
                    verify_ssl=False,
                ) as response:
                    data = await response.json(content_type="application/json")
        except Exception as ex:
            logger.debug(ex)
            logger.error("There was something wrong with your request.")
            return False
        if response.status in [200, 201, 204]:
            logger.info("Post successful.")
            return data
        if response.status == 404:
            logger.error("Resource not found: %s" % self.url + endpoint)
        return False

    async def put_request(self, endpoint, payload):
        logger.debug("POST: {%s:%s}" % (endpoint, payload))
        try:
            async with aiohttp.ClientSession(
                headers=self.headers, loop=self.loop
            ) as session:
                async with session.put(
                    self.url + endpoint,
                    json=payload,
                    verify_ssl=False,
                ) as response:
                    await response.json(content_type="application/json")
        except Exception as ex:
            logger.debug(ex)
            logger.error("There was something wrong with your request.")
            return False
        if response.status in [200, 201, 204]:
            logger.info("Post successful.")
            return True
        if response.status == 404:
            logger.error("Resource not found: %s" % self.url + endpoint)
        return False

    async def create_ticket(self, summary, description):
        endpoint = "/issue/"
        logger.debug("POST new ticket")
        short_summary = summary.split('\r')
        title = f"{short_summary[0]}"

        data = {
            "fields": {
                "project": {"key": self.ticket_queue},
                "issuetype": {"name": "Task"},
                "summary": title,
                "description": description,
            }
        }
        response = await self.post_request(endpoint, data)
        return response

    async def create_subtask(self, parent_ticket, cloud, description, type_of_subtask):
        endpoint = "/issue/"
        logger.debug("POST new subtask")
        title = f"{cloud} {type_of_subtask}"

        data = {
            "fields": {
                "project": {"key": self.ticket_queue},
                "issuetype": {"id": "5"},
                "parent": {"key": f"{self.ticket_queue}-{parent_ticket}"},
                "summary": title,
                "labels": [type_of_subtask.upper()],
                "description": description,
            }
        }
        response = await self.post_request(endpoint, data)
        return response

    async def add_watcher(self, ticket, watcher):
        issue_id = "%s-%s" % (self.ticket_queue, ticket)
        endpoint = "/issue/%s/watchers" % issue_id
        logger.debug("POST transition: {%s:%s}" % (issue_id, watcher))
        response = await self.post_request(endpoint, watcher)
        return response

    async def add_label(self, ticket, label):
        issue_id = "%s-%s" % (self.ticket_queue, ticket)
        endpoint = "/issue/%s" % issue_id
        data = {"update": {"labels": [{"add": label}]}}
        response = await self.put_request(endpoint, data)
        return response

    async def post_comment(self, ticket, comment):
        issue_id = "%s-%s" % (self.ticket_queue, ticket)
        endpoint = "/issue/%s/comment" % issue_id
        payload = {"body": comment}
        response = await self.post_request(endpoint, payload)
        return response

    async def post_transition(self, ticket, transition):
        issue_id = "%s-%s" % (self.ticket_queue, ticket)
        endpoint = "/issue/%s/transitions" % issue_id
        payload = {"transition": {"id": transition}}
        response = await self.post_request(endpoint, payload)
        return response

    async def get_transitions(self, ticket):
        issue_id = "%s-%s" % (self.ticket_queue, ticket)
        endpoint = "/issue/%s/transitions" % issue_id
        result = await self.get_request(endpoint)
        if not result:
            logger.error("Failed to get transitions")
            return []

        transitions = result.get("transitions")
        if transitions:
            return transitions
        else:
            logger.error("No transitions found under %s" % issue_id)
            return []

    async def get_ticket(self, ticket):
        issue_id = "%s-%s" % (self.ticket_queue, ticket)
        endpoint = "/issue/%s" % issue_id
        result = await self.get_request(endpoint)
        if not result:
            logger.error("Failed to get ticket")
            return None
        return result

    async def get_watchers(self, ticket):
        issue_id = "%s-%s" % (self.ticket_queue, ticket)
        endpoint = "/issue/%s/watchers" % issue_id
        logger.debug("GET watchers: %s" % endpoint)
        result = await self.get_request(endpoint)
        if not result:
            logger.error("Failed to get watchers")
            return None
        return result

    async def get_user_by_email(self, email):
        endpoint = f"/user/search?username={email}"
        logger.debug("GET user: %s" % endpoint)
        result = await self.get_request(endpoint)
        if not result:
            logger.error("User not found")
            return None
        for user in result:
            if user.get("emailAddress") == email:
                return user
        return None

    async def get_pending_tickets(self):
        query = {"statusCategory": 4, "labels": "EXTENSION"}
        logger.debug("GET pending tickets")
        result = await self.search_tickets(query)
        if not result:
            logger.error("Failed to get pending tickets")
            return None
        return result

    async def search_tickets(self, query=None):
        project = {"project": self.ticket_queue}
        prefix = "/search?jql="
        query_items = []

        if not query:
            query = project
        else:
            query.update(project)

        for k, v in query.items():
            query_items.append(f"{k}={v}")

        jql = " AND ".join(query_items)
        endpoint = f"{prefix}{jql}"
        logger.debug("GET pending tickets: %s" % endpoint)
        result = await self.get_request(endpoint)
        if not result:
            logger.error("Failed to get pending tickets")
            return None
        return result



from datetime import datetime

from cloud_governance.common.logger.init_logger import logger
from cloud_governance.policy.policy_operations.aws.tag_non_cluster.non_cluster_operations import NonClusterOperations


class RemoveNonClusterTags(NonClusterOperations):

    def __init__(self, region: str = 'us-east-2', dry_run: str = 'yes', input_tags: dict = ''):
        super().__init__(region=region, dry_run=dry_run, input_tags=input_tags)

    def __get_instance_tags(self, launch_time: datetime, instance_id: str, tags: list):
        """
        This method returns the tags to update  the instance tags
        @param launch_time:
        @param instance_id:
        @param tags:
        @return:
        """
        username = self._get_username_from_cloudtrail(start_time=launch_time, resource_id=instance_id, resource_type='AWS::EC2::Instance')
        search_tags = []
        user_tags = self.iam_client.get_user_tags(username=username)
        if not username:
            search_tags.extend(self._fill_na_tags())
        else:
            search_tags.append(self._build_tag(key='Email', value=f'{username}@redhat.com'))
        search_tags.append(self._build_tag(key='LaunchTime', value=launch_time))
        search_tags.extend(self._append_input_tags())
        search_tags.extend(user_tags)
        return search_tags

    def non_cluster_update_ec2(self, instances_list: list = None):
        """
        This method removes the tags of ec2 instances
        @return:
        """
        if not instances_list:
            instances_list = self._get_resource_data(resource_method=self._get_instances_data)
        instances_ids = []
        for instance in instances_list:
            for item in instance:
                instance_id = item.get('InstanceId')
                launch_time = item.get('LaunchTime')
                add_tags = self.__get_instance_tags(launch_time=launch_time, instance_id=instance_id, tags=item.get('Tags'))
                if add_tags:
                    if self.dry_run == 'no':
                        self.ec2_client.delete_tags(Resources=[instance_id], Tags=add_tags)
                        logger.info(f'Delete tags of instance: {instance_id} total: {len(add_tags)} tags: {add_tags}')
                    instances_ids.append(instance_id)
        logger.info(f'non_cluster_ec2 count: {len(sorted(instances_ids))} {sorted(instances_ids)}')
        return sorted(instances_ids)

    def update_volumes(self, volumes_data: list = None):
        """
        This method removes the tags of non-cluster volumes
        @param volumes_data:
        @return:
        """
        if not volumes_data:
            volumes_data = self._get_resource_data(resource_method=self.ec2_operations.get_volumes)
        volume_ids = []
        for volume in volumes_data:
            volume_id = volume.get('VolumeId')
            tag_name = ''
            username = self._get_username_from_cloudtrail(start_time=volume.get('CreateTime'), resource_id=volume_id, resource_type='AWS::EC2::Volume')
            search_tags = []
            if not username:
                get_tags, username = self._get_tags_fom_attachments(attachments=volume.get('Attachments'))
                search_tags.extend(get_tags)
            else:
                search_tags.extend(self._append_input_tags())
            if username and not tag_name:
                user_tags = self.iam_client.get_user_tags(username=username)
                search_tags.extend(user_tags)
                search_tags.append({'Key': 'Email', 'Value': f'{username}@redhat.com'})
                search_tags.append(self._build_tag(key='LaunchTime', value=volume.get('CreateTime')))
            else:
                search_tags.extend(self._fill_na_tags())
                search_tags.extend(self._append_input_tags())
                search_tags.append(self._build_tag(key='LaunchTime', value=volume.get('CreateTime')))
            if search_tags:
                if self.dry_run == 'no':
                    self.ec2_client.delete_tags(Resources=[volume_id], Tags=search_tags)
                    logger.info(f'Remove tags of volume_id: {volume_id} total: {len(search_tags)}  tags: {search_tags}')
                volume_ids.append(volume_id)
        logger.info(f'non_cluster_volumes count: {len(sorted(volume_ids))} {sorted(volume_ids)}')
        return sorted(volume_ids)

    def update_snapshots(self, snapshots: list = None):
        """
        This method removes the tags of  non-cluster snapshots
        @param snapshots:
        @return:
        """
        if not snapshots:
            snapshots = self._get_resource_data(resource_method=self.ec2_operations.get_snapshots)
        snapshot_ids = []
        for snapshot in snapshots:
            snapshot_id = snapshot.get('SnapshotId')
            username = self._get_username_from_cloudtrail(start_time=snapshot.get('StartTime'), resource_id=snapshot_id, resource_type='AWS::EC2::Snapshot')
            search_tags = []
            if not username:
                if snapshot.get('Description') and 'Created' in snapshot.get('Description'):
                    image_tags, username = self._get_tags_from_snapshot_description_images(description=snapshot.get('Description'))
                    if not username:
                        instance_id = snapshot.get('Description').split(" ")[2].split("(")[1][:-1]
                        instances = self._get_instances_data(instance_id)
                        if instances:
                            for item in instances:
                                if item.get('InstanceId') == instance_id:
                                    item_tags, username = self._get_tags_from_instance_item(instance_item=item)
            else:
                search_tags.extend(self._append_input_tags())
            if username:
                user_tags = self.iam_client.get_user_tags(username=username)
                search_tags.extend(user_tags)
                search_tags.append({'Key': 'Email', 'Value': f'{username}@redhat.com'})
            else:
                search_tags.extend(self._fill_na_tags())
                search_tags.extend(self._append_input_tags())
            search_tags.append(self._build_tag(key='LaunchTime', value=snapshot.get('StartTime')))
            if search_tags:
                if self.dry_run == 'no':
                    self.ec2_client.delete_tags(Resources=[snapshot_id], Tags=search_tags)
                    logger.info(
                        f'Remove tags of snapshots: {snapshot_id} total: {len(search_tags)} tags: {search_tags}')
                snapshot_ids.append(snapshot_id)
        logger.info(f'non_cluster_snapshot count: {len(sorted(snapshot_ids))} {sorted(snapshot_ids)}')
        return sorted(snapshot_ids)

    def update_ami(self, images: list = None):
        """
        This method removes the tags of non-cluster Amazon Machine Images
        @param images:
        @return:
        """
        if not images:
            images = self.ec2_operations.get_images()
            _, images = self.ec2_operations.scan_cluster_non_cluster_resources(images)
        image_ids = []
        for image in images:
            image_id = image.get('ImageId')
            start_time = datetime.fromisoformat(image.get('CreationDate')[:-1] + '+00:00')
            username = self._get_username_from_cloudtrail(start_time=start_time, resource_id=image_id, resource_type='AWS::EC2::Ami')
            search_tags = []
            search_tags.extend(self._append_input_tags())
            if username:
                user_tags = self.iam_client.get_user_tags(username=username)
                search_tags.extend(user_tags)
                search_tags.append({'Key': 'Email', 'Value': f'{username}@redhat.com'})
            else:
                search_tags.extend(self._fill_na_tags())
            search_tags.append(self._build_tag(key='LaunchTime', value=start_time))
            if search_tags:
                if self.dry_run == 'no':
                    self.ec2_client.delete_tags(Resources=[image_id], Tags=search_tags)
                    logger.info(f'Remove tags of image: {image_id} total: {len(search_tags)} tags: {search_tags}')
                image_ids.append(image_id)
        logger.info(f'non_cluster_amis count: {len(sorted(image_ids))} {sorted(image_ids)}')
        return sorted(image_ids)




from cloud_governance.main.environment_variables import environment_variables


class IBMOperations:
    """
    This class contain the common parameters of ibm polices
    """

    def __init__(self):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self._account = self.__environment_variables_dict.get('account', '')
        self._dry_run = self.__environment_variables_dict.get('dry_run', 'yes')
        self._policy = self.__environment_variables_dict.get('policy', '')
        self._tag_operation = self.__environment_variables_dict.get('tag_operation', 'read')







import time

import boto3
import typeguard
from botocore.client import BaseClient

from cloud_governance.common.clouds.aws.ec2.ec2_operations import EC2Operations
from cloud_governance.common.clouds.aws.utils.utils import Utils
from cloud_governance.common.logger.init_logger import logger


class DeleteEC2Resources:
    """
    This class deletes the EC2 resources
    0 dependencies
    Elastic Load Balancer
    Elastic Load Balancer v2
    volumes
    snapshots
    VPC Endpoints
    DHCP options set
    Route Table
    Security Groups
    NatGateway

    1 Dependencies
    NetworkACL [ Subnet ]

    2 Dependencies
    Network Interface [ NatGateway, Elastic Load Balancer ]
    Internet Gateway [ NatGateway, ElasticIP ]

    3 Dependencies
    Subnet [ Network Interface, NatGateway, Elastic Load Balancer ]
    Elastic Ip [ Network interface, NatGateway, Elastic Load Balancer ]

    12 Dependencies
    VPC [ Network Interface, NatGateway, Elastic Load Balancer, Vpc Endpoints, Route tables, Subnets,
          Security Groups, NetworkACl]
    """

    SLEEP_TIME = 30

    def __init__(self, client: BaseClient, elb_client: BaseClient, elbv2_client: BaseClient, region: str = 'us-east-2'):
        self.cluster_tag = None
        self.client = client
        self.elb_client = elb_client
        self.elbv2_client = elbv2_client
        self.get_detail_list = Utils().get_details_resource_list
        self.ec2_operations = EC2Operations(region=region)
        self.efs_client = boto3.client('efs', region_name=region)

    @typeguard.typechecked
    def delete_zombie_resource(self, resource: str, resource_id: str,  cluster_tag: str = '', vpc_id: str = '',
                               deletion_type: str = '', pending_resources: dict = ''):
        """
        This method filter the resource and call to delete resource function
        :param pending_resources:
        :param delete_elip:
        :param deletion_type:
        :param resource:
        :param resource_id:
        :param cluster_tag:
        :param vpc_id:
        :return:
        """
        self.cluster_tag = cluster_tag
        if resource == 'load_balancer':
            self.__delete_load_balancer(resource_id=resource_id)
        elif resource == 'load_balancer_v2':
            self.__delete_load_balancer_v2(resource_id=resource_id)
        elif resource == 'ec2_volume':
            self.__delete_volume(resource_id=resource_id)
        elif resource == 'ebs_snapshots':
            self.__delete_snapshots(resource_id=resource_id)
        elif resource == 'vpc_endpoints':
            self.__delete_vpc_endpoints(resource_id=resource_id)
        elif resource == 'dhcp_options':
            self.__delete_dhcp_options(resource_id=resource_id, vpc_id=vpc_id)
        elif resource == 'route_table':
            self.__delete_route_table(resource_id=resource_id, vpc_id=vpc_id)
        elif resource == 'security_group':
            self.__delete_security_group(resource_id=resource_id, vpc_id=vpc_id)
        elif resource == 'nat_gateways':
            self.__delete_nat_gateways(resource_id=resource_id)
        elif resource == 'network_acl':
            self.__delete_network_acl(resource_id=resource_id, vpc_id=vpc_id)
        elif resource == 'network_interface':
            self.__delete_network_interface(resource_id=resource_id)
        elif resource == 'internet_gateway':
            self.__delete_internet_gateway(resource_id=resource_id, vpc_id=vpc_id)
        elif resource == 'subnet':
            self.__delete_subnet(resource_id=resource_id)
        elif resource == 'elastic_ip':
            self.__delete_elastic_ip(resource_id=resource_id, deletion_type=deletion_type)
        elif resource == 'vpc':
            self.__delete_vpc(resource_id=resource_id, pending_resources=pending_resources)

    @typeguard.typechecked
    def __get_cluster_references(self, resource_id: str, resource_list: list,
                                 input_resource_id: str, output_result: str):
        """
        This method get the cluster resources based on input format and output format
        :param resource_id:
        :param resource_list:
        :param input_resource_id:
        :param output_result:
        :return:
        """
        result = []
        for resource in resource_list:
            if output_result in resource:
                if input_resource_id in resource and resource.get(input_resource_id) == resource_id:
                    if isinstance(resource.get(output_result), list):
                        result.extend(resource.get(output_result))
                    else:
                        result.append(resource.get(output_result))
            elif output_result == '':
                if input_resource_id in resource and resource.get(input_resource_id) == resource_id:
                    result.append(resource)
        return result

    @typeguard.typechecked
    def __delete_load_balancer(self, resource_id: str):
        """
        Delete the Load Balancer based on LoadBalancer ID
        :param resource_id:
        :return:
        """
        load_balancers = self.ec2_operations.get_load_balancers()
        for load_balancer in load_balancers:
            if load_balancer.get('LoadBalancerName') == resource_id:
                tags = self.elb_client.describe_tags(LoadBalancerNames=[resource_id])
                if tags['TagDescriptions']:
                    for item in tags['TagDescriptions']:
                        if item.get('Tags'):
                            if self.__is_cluster_resource(item.get('Tags'), self.cluster_tag):
                                try:
                                    self.elb_client.delete_load_balancer(LoadBalancerName=resource_id)
                                    logger.info(f'delete_load_balancer: {resource_id}')
                                except Exception as err:
                                    logger.exception(f'Cannot delete_load_balancer: {resource_id}, {err}')

    @typeguard.typechecked
    def __delete_load_balancer_v2(self, resource_id: str):
        """
        Delete the Load Balancer based on LoadBalancer ID
        :param resource_id:
        :return:
        """
        load_balances = self.elbv2_client.describe_load_balancers()['LoadBalancers']
        for load_balancer in load_balances:
            if load_balancer.get('LoadBalancerArn').endswith(resource_id):
                tags = self.elbv2_client.describe_tags(ResourceArns=[load_balancer.get('LoadBalancerArn')])
                if tags.get('TagDescriptions'):
                    for item in tags['TagDescriptions']:
                        if item.get('Tags'):
                            if self.__is_cluster_resource(item.get('Tags'), self.cluster_tag):
                                try:
                                    self.elbv2_client.delete_load_balancer(LoadBalancerArn=load_balancer.get('LoadBalancerArn'))
                                    logger.info(f'delete_load_balancer: {resource_id}')
                                except Exception as err:
                                    logger.exception(f'Cannot delete_load_balancer: {resource_id}, {err}')

    @typeguard.typechecked
    def __delete_volume(self, resource_id: str):
        try:
            logger.info(f'Cluster volumes are handled by ebs_unattached')
            # self.client.delete_volume(VolumeId=resource_id)
            # logger.info(f'delete_volume: {resource_id}')
        except Exception as err:
            logger.exception(f'Cannot delete_volume: {resource_id}, {err}')

    @typeguard.typechecked
    def __delete_snapshots(self, resource_id: str):
        try:
            self.client.delete_snapshot(SnapshotId=resource_id)
            logger.info(f'delete_snapshot: {resource_id}')
        except Exception as err:
            logger.exception(f'Cannot delete_snapshot: {resource_id}, {err}')

    @typeguard.typechecked
    def __delete_vpc_endpoints(self, resource_id: str):
        """
        This method delete the VPC endpoints based on vpc_endpoint ID
        :param resource_id:
        :return:
        """
        try:
            self.client.delete_vpc_endpoints(VpcEndpointIds=[resource_id])
            logger.info(f'delete_vpc_endpoints: {resource_id}')
        except Exception as err:
            logger.exception(f'Cannot delete_vpc_endpoints: {resource_id}, {err}')

    @typeguard.typechecked
    def __delete_dhcp_options(self, resource_id: str, vpc_id: str = ''):
        """
        This method delete the dhcp options in the following order
        set associate dhcp options in vpc as default --> delete dhcp options
        :param resource_id:
        :return:
        """
        try:
            if vpc_id:
                self.client.associate_dhcp_options(DhcpOptionsId='default', VpcId=vpc_id)
            self.client.delete_dhcp_options(DhcpOptionsId=resource_id)
            logger.info(f'delete_dhcp_options: {resource_id}')
        except Exception as err:
            logger.exception(f'Cannot delete_dhcp_options: {resource_id}, {err}')

    @typeguard.typechecked
    def __delete_route_table(self, resource_id: str, vpc_id: str):
        """
        This method deleted the Route Table in the following order
        Disassociate Route Tables subnets --> Route table
        :param resource_id:
        :return:
        """
        # @todo call vpc deletion
        try:
            route_tables = self.ec2_operations.get_route_tables()
            subnets = self.__get_cluster_references(resource_id=resource_id, resource_list=route_tables,
                                                    input_resource_id='RouteTableId',
                                                    output_result='Associations')
            main_route_table = False
            for subnet in subnets:
                if subnet.get('SubnetId'):
                    self.client.disassociate_route_table(AssociationId=subnet['RouteTableAssociationId'])
                elif subnet.get('RouteTableId') and not subnet.get('Main'):
                    self.client.disassociate_route_table(AssociationId=subnet['RouteTableAssociationId'])
                if subnet.get('Main'):
                    main_route_table = True
            if not main_route_table:
                self.client.delete_route_table(RouteTableId=resource_id)
                logger.info(f'delete_route_table: {resource_id}')
            else:
                logger.info(f'Main route table: {resource_id} is deleted by vpc: {vpc_id} ')
        except Exception as err:
            logger.exception(f'Cannot delete_route_table: {resource_id}, {err}')

    def __is_cluster_resource(self, tags: list, cluster_tag):
        """
        This is method check resource is Cluster or not
        @return:
        """
        if cluster_tag:
            for tag in tags:
                if cluster_tag == tag.get('Key'):
                    return True
        return False

    @typeguard.typechecked
    def __delete_security_group(self, resource_id: str, vpc_id: str):
        """
        This method deletes the NatGateway in the following order
        remove security group ingress --> modify security groups in Network interface --> delete security group
        :param resource_id:
        :return:
        """
        try:
            security_groups = self.ec2_operations.get_security_groups()
            vpc_security_groups = self.__get_cluster_references(resource_id=vpc_id, resource_list=security_groups, input_resource_id='VpcId', output_result='')
            for vpc_security_group in vpc_security_groups:
                if resource_id != vpc_security_group.get('GroupId'):
                    if vpc_security_group.get('GroupName') == 'default':
                        logger.info(f'Removing the {resource_id} ingress rule from Default Security Group: {vpc_security_group.get("GroupId")}')
                        if vpc_security_group.get('IpPermissions'):
                            for ip_permission in vpc_security_group.get('IpPermissions'):
                                if ip_permission.get('UserIdGroupPairs'):
                                    for user_id_group_pair in ip_permission.get('UserIdGroupPairs'):
                                        if user_id_group_pair.get('GroupId') == resource_id:
                                            ingress_rule = {'FromPort': ip_permission.get('FromPort'), 'IpProtocol': ip_permission.get('IpProtocol'), 'IpRanges': ip_permission.get('IpRanges'), 'Ipv6Ranges': ip_permission.get('Ipv6Ranges'), 'PrefixListIds': ip_permission.get('PrefixListIds'), 'ToPort': ip_permission.get('ToPort'), 'UserIdGroupPairs': [user_id_group_pair]}
                                            self.client.revoke_security_group_ingress(GroupId=vpc_security_group.get('GroupId'), IpPermissions=[ingress_rule])
                                            logger.info(f'Removed the Ingress rules of Security Group {vpc_security_group.get("GroupId")} :: {ingress_rule}')
                    else:
                        if vpc_security_group.get('Tags'):
                            if self.__is_cluster_resource(tags=vpc_security_group.get('Tags'), cluster_tag=self.cluster_tag):
                                logger.info(vpc_security_group.get('GroupId'))
                                if vpc_security_group.get('IpPermissions'):
                                    for ip_permission in vpc_security_group.get('IpPermissions'):
                                        if ip_permission.get('UserIdGroupPairs'):
                                            for user_id_group_pair in ip_permission.get('UserIdGroupPairs'):
                                                if user_id_group_pair.get('GroupId') == resource_id:
                                                    ingress_rule = {'FromPort': ip_permission.get('FromPort'), 'IpProtocol': ip_permission.get('IpProtocol'), 'IpRanges': ip_permission.get('IpRanges'),
                                                                    'Ipv6Ranges': ip_permission.get('Ipv6Ranges'), 'PrefixListIds': ip_permission.get('PrefixListIds'),
                                                                    'ToPort': ip_permission.get('ToPort'), 'UserIdGroupPairs': [user_id_group_pair]}
                                                    self.client.revoke_security_group_ingress(GroupId=vpc_security_group.get('GroupId'), IpPermissions=[ingress_rule])
                                                    logger.info(f'Removed the Ingress rules of Security Group {resource_id} from {ingress_rule}')
            network_interfaces = self.ec2_operations.get_network_interface()
            network_interface_ids = self.__get_cluster_references(resource_id=vpc_id, resource_list=network_interfaces,
                                                                  input_resource_id='VpcId',
                                                                  output_result='')
            default_security_group_id = [security_group.get('GroupId') for security_group in vpc_security_groups
                                         if security_group.get('GroupName') == 'default']
            if default_security_group_id:
                default_security_group_id = default_security_group_id[0]
            for network_interface in network_interface_ids:
                if network_interface.get('TagSet'):
                    if self.__is_cluster_resource(network_interface.get('TagSet'), cluster_tag=self.cluster_tag):
                        logger.info(network_interface.get('NetworkInterfaceId'))
                        for security_group in network_interface.get('Groups'):
                            if security_group.get('GroupId') == resource_id and default_security_group_id != security_group.get('GroupId'):
                                self.client.modify_network_interface_attribute(Groups=[default_security_group_id], NetworkInterfaceId=network_interface.get('NetworkInterfaceId'))
            if resource_id == default_security_group_id:
                logger.info(f'default security group: {resource_id} is deleted by vpc: {vpc_id} ')
            else:
                self.client.delete_security_group(GroupId=resource_id)
                logger.info(f'delete_security_group: {resource_id}')
        except Exception as err:
            logger.exception(f'Cannot delete_security_group: {resource_id}, {err}')

    @typeguard.typechecked
    def __delete_nat_gateways(self, resource_id: str):
        """
        This method delete the NatGateway based on NatGateway ID
        :param resource_id:
        :return:
        """
        try:
            nat_gateway = self.client.describe_nat_gateways(Filter=[{'Name': 'nat-gateway-id', 'Values': [resource_id]}])['NatGateways'][0]
            if nat_gateway.get('State') == 'available':
                self.client.delete_nat_gateway(NatGatewayId=resource_id)
                while nat_gateway.get('State') != 'deleted':
                    nat_gateway = self.client.describe_nat_gateways(Filter=[{'Name': 'nat-gateway-id', 'Values': [resource_id]}])['NatGateways'][0]
                    time.sleep(self.SLEEP_TIME)
                logger.info(f'delete_nat_gateway: {resource_id}')
        except Exception as err:
            logger.exception(f'Cannot delete_nat_gateway: {resource_id}, {err}')

    @typeguard.typechecked
    def __delete_network_acl(self, resource_id, vpc_id: str):
        try:
            nacls = self.ec2_operations.get_nacls()
            vpc_nacls = self.__get_cluster_references(resource_id=resource_id, resource_list=nacls,
                                                      input_resource_id='NetworkAclId', output_result='Associations')
            default_nacl = self.__get_cluster_references(resource_id=resource_id, resource_list=nacls,
                                                         input_resource_id='NetworkAclId', output_result='IsDefault')
            if default_nacl:
                default_nacl = default_nacl[0]
                for vpc_nacl in vpc_nacls:
                    self.__delete_subnet(resource_id=vpc_nacl['SubnetId'])
                if not default_nacl:
                    self.client.delete_network_acl(NetworkAclId=resource_id)
                    logger.info(f'delete_network_acl: {resource_id}')
                else:
                    logger.info(f'default network acl: {resource_id} is deleted by vpc: {vpc_id} ')
        except Exception as err:
            logger.exception(f'Cannot delete_network_acl: {resource_id}, {err}')

    @typeguard.typechecked
    def __delete_network_interface(self, resource_id: str):
        """
        This method deletes the Network Interface in the following order
        Load balancer / NatGateway --> Release Elastic IPs --> detach Network Interface --> Delete Network Interface
        :param resource_id:
        :return:
        """
        try:
            resource_list = self.ec2_operations.get_network_interface()
            descriptions = self.__get_cluster_references(resource_id=resource_id, resource_list=resource_list,
                                                         input_resource_id='NetworkInterfaceId',
                                                         output_result="Description")
            delete = False
            if descriptions:
                for description in descriptions:
                    if "ELB" in description:
                        loadbalancer = description.split(" ")[-1]
                        if '/' in loadbalancer:
                            self.__delete_load_balancer_v2(loadbalancer)
                        else:
                            self.__delete_load_balancer(loadbalancer)
                    elif "NAT" in description:
                        self.__delete_nat_gateways(resource_id=description.split(" ")[-1])
                        delete = True
                    elif 'EFS' in description:
                        self.__delete_efs(resource_id=description.split(" ")[-2])
                        delete = True
            if resource_list and not delete:
                attachments = self.__get_cluster_references(resource_id=resource_id, resource_list=resource_list,
                                                            input_resource_id='NetworkInterfaceId',
                                                            output_result='Attachment')
                if attachments:
                    self.client.detach_network_interface(AttachmentId=attachments[0]['AttachmentId'])
                self.client.delete_network_interface(NetworkInterfaceId=resource_id)
                logger.info(f'delete_network_interface: {resource_id}')
        except Exception as err:
            logger.exception(f'Cannot disassociate_address: {resource_id}, {err}')

    def __delete_efs(self, resource_id: str):
        """
        This method deletes the EFS file system
        @param resource_id:
        @return:
        """
        efs_data = self.efs_client.describe_file_systems()['FileSystems']

        for efs in efs_data:
            if efs.get('FileSystemId') == resource_id:
                efsmt_data = self.efs_client.describe_mount_targets(FileSystemId=resource_id)['MountTargets']
                for efsmt in efsmt_data:
                    self.efs_client.delete_mount_target(MountTargetId=efsmt.get('MountTargetId'))
                    logger.info(f'deleted Efs Mount target {efsmt.get("MountTargetId")}')
                try:
                    self.efs_client.delete_file_system(FileSystemId=efs.get('FileSystemId'))
                except Exception as err:
                    logger.info(f'{err}')
                logger.info(f'Deleted the EFS file system {efs.get("FileSystemId")}')

    @typeguard.typechecked
    def network_interface(self, subnet_id: str = ''):
        """
        This method calls delete interface method if subnet is used by network interface
        :param subnet_id:
        :return:
        """
        network_interfaces = self.ec2_operations.get_network_interface()
        network_interface_ids = self.__get_cluster_references(resource_id=subnet_id,
                                                              resource_list=network_interfaces,
                                                              input_resource_id='SubnetId',
                                                              output_result="NetworkInterfaceId")
        for network_interface_id in network_interface_ids:
            if subnet_id:
                self.__delete_network_interface(resource_id=network_interface_id)

    @typeguard.typechecked
    def __delete_internet_gateway(self, resource_id: str, vpc_id: str):
        """
        This method delete Internet gateway in the following order
        NatGateway( Delete Network Interface associated with It ) --> Release Address --> Internet Gateway
        :param resource_id:
        :param vpc_id:
        :return:
        """
        try:
            network_interfaces = self.ec2_operations.get_network_interface()
            network_interface_ids = self.__get_cluster_references(resource_id=vpc_id, resource_list=network_interfaces,
                                                                  input_resource_id='VpcId',
                                                                  output_result='NetworkInterfaceId')

            for network_interface_id in network_interface_ids:
                self.__delete_network_interface(resource_id=network_interface_id)
            if vpc_id:
                self.client.detach_internet_gateway(InternetGatewayId=resource_id, VpcId=vpc_id)
            self.client.delete_internet_gateway(InternetGatewayId=resource_id)
            logger.info(f'delete_internet_gateway: {resource_id}')
        except Exception as err:
            logger.exception(f'Cannot delete_internet_gateway: {resource_id}, {err}')

    @typeguard.typechecked
    def __delete_subnet(self, resource_id: str):
        """
        This method delete the Subnets in the following order
        Network Interface --> Subnet
        :param resource_id:
        :return:
        """
        try:
            self.network_interface(subnet_id=resource_id)
            self.client.delete_subnet(SubnetId=resource_id)
            logger.info(f'delete_subnet: {resource_id}')
        except Exception as err:
            logger.exception(f'Cannot delete_subnet: {resource_id}, {err}')

    @typeguard.typechecked
    def __delete_elastic_ip(self, resource_id: str, deletion_type: str):
        """
        This method delete Elastic Ip in the following order
        Network Interface --> Release IP
        :param resource_id:
        :return:
        """
        try:
            if deletion_type:
                elastic_ips = self.ec2_operations.get_elastic_ips()
                network_interfaces = self.__get_cluster_references(resource_id=resource_id, resource_list=elastic_ips,
                                                                   input_resource_id='AssociationId',
                                                                   output_result='')
                for network_interface in network_interfaces:
                    if network_interface.get('TagSet'):
                        if self.__is_cluster_resource(network_interface.get('TagSet'), self.cluster_tag):
                            self.__delete_network_interface(network_interface.get('NetworkInterfaceId'))
                        else:
                            logger.info(f'This network interface Id : {network_interface.get("NetworkInterfaceId")} not a zombie cluster resource')
            else:
                self.client.release_address(AllocationId=resource_id)
                logger.info(f'release_address: {resource_id}')
        except Exception as err:
            logger.exception(f'Cannot release_address: {resource_id}, {err}')

    @typeguard.typechecked
    def __delete_vpc(self, resource_id: str, pending_resources: dict):
        """
        This method delete the vpc in the following order
        NatGateway -> Network Interface -> Security groups --> VPC Endpoints --> Route Tables --> Network Acls --> vpc
        :param resource_id:
        :param pending_resources:
        :return:
        """
        try:
            i = 0
            for key, pending_resource in pending_resources.items():
                pending_resource(resource_id, self.cluster_tag)
            vpc_peerings = self.client.describe_vpc_peering_connections()['VpcPeeringConnections']
            for vpc_peering in vpc_peerings:
                if vpc_peering.get('Status').get('Code') == 'active':
                    if vpc_peering.get('RequesterVpcInfo').get('VpcId') == resource_id:
                        self.client.delete_vpc_peering_connection(VpcPeeringConnectionId=vpc_peering.get('VpcPeeringConnectionId'))
                    elif vpc_peering.get('AccepterVpcInfo').get('VpcId') == resource_id:
                        self.client.delete_vpc_peering_connection(VpcPeeringConnectionId=vpc_peering.get('VpcPeeringConnectionId'))
                elif vpc_peering.get('Status').get('Code') == 'pending-acceptance':
                    if vpc_peering.get('RequesterVpcInfo').get('VpcId') == resource_id:
                        self.client.delete_vpc_peering_connection(VpcPeeringConnectionId=vpc_peering.get('VpcPeeringConnectionId'))
            self.client.delete_vpc(VpcId=resource_id)
            logger.info(f'delete_vpc: {resource_id}')
        except Exception as err:
            logger.exception(f'Cannot delete_vpc: {resource_id}, {err}')



import importlib
import inspect

from cloud_governance.common.logger.init_logger import logger
from cloud_governance.main.environment_variables import environment_variables


class CostReportPolicies:

    def __init__(self):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self._account = self.__environment_variables_dict.get('account', '')
        self._policy = self.__environment_variables_dict.get('policy', '')

    def run(self):
        """
        This method run the AWS cost policies
        @return:
        """
        logger.info(f'account={self._account}, policy={self._policy}')
        cost_report_policy_module = importlib.import_module(f'cloud_governance.policy.aws.{self._policy}')
        for cls in inspect.getmembers(cost_report_policy_module, inspect.isclass):
            if self._policy.replace('_', '') == cls[0].lower():
                response = cls[1]().run()
                if response:
                    logger.info(f'key: {cls[0]}, count: {len(response)}, {response}')



from abc import ABC, abstractmethod
from datetime import datetime

from cloud_governance.main.environment_variables import environment_variables


class AbstractAthenaOperations(ABC):

    CURRENT_DATE = str(datetime.utcnow().date()).replace("-", "")

    def __init__(self):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.__s3_results_path = self.__environment_variables_dict.get('S3_RESULTS_PATH')
        super().__init__()

    def _get_s3_path(self):
        """
        This method returns the s3 path to dump athena results
        :return:
        """
        s3_path = f"{self.__s3_results_path}/{self.CURRENT_DATE}"
        return s3_path

    @abstractmethod
    def execute_query(self, query_string: str):
        """
        This method executes the query in aws athena
        :param query_string:
        :return:
        """
        raise NotImplemented()




import typeguard
from pyathena import connect
from pyathena.pandas.cursor import PandasCursor

from cloud_governance.common.clouds.aws.athena.abstract_athena_operations import AbstractAthenaOperations
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp


class PyAthenaOperations(AbstractAthenaOperations):

    def __init__(self):
        super().__init__()
        self.conn = connect(s3_staging_dir=self._get_s3_path(), region_name="us-east-1",
                            cursor_class=PandasCursor).cursor()

    @typeguard.typechecked
    @logger_time_stamp
    def execute_query(self, query_string: str) -> list:
        """
        This method returns list of dicts of athena results
        :param query_string:
        :return:
        """
        try:
            result_set = self.conn.execute(query_string).as_pandas()
            return result_set.to_dict(orient='records')
        except Exception as err:
            raise err




class EnvironmentVariablesExceptions(Exception):
    """ Base class for all environment variables error classes.
        All exceptions raised by the benchmark runner library should inherit from this class. """
    pass


class ParseFailed(EnvironmentVariablesExceptions):
    """Unparseable environment variable"""
    def __init__(self, message):
        self.message = message
        super(ParseFailed, self).__init__(self.message)



import os
import sys
import logging

from cloud_governance.main.environment_variables import environment_variables

logger_category_name = 'cloud_governance'
logger = logging.getLogger(logger_category_name)  # instantiating a logger
handler = logging.StreamHandler(sys.stdout)
# log for output only
#log_path = os.getcwd()
#fileHandler = logging.FileHandler(filename=f'{log_path}/cloud_governance.log', mode='w+')
account_name = environment_variables.environment_variables_dict.get('account')
log_format = f'[%(levelname)s] %(asctime)s {account_name} - %(message)s'
formatter = logging.Formatter(log_format)
handler.setFormatter(formatter)
logger.addHandler(handler)
#logger.addHandler(fileHandler)


# def get_pyperf_log_path():
#     return f'{log_path}/py_perf.log'




from abc import abstractmethod, ABC
from datetime import datetime

import typeguard

from cloud_governance.cloud_resource_orchestration.utils.common_operations import string_equal_ignore_case
from cloud_governance.cloud_resource_orchestration.utils.elastic_search_queries import ElasticSearchQueries
from cloud_governance.cloud_resource_orchestration.utils.constant_variables import FIRST_CRO_ALERT, SECOND_CRO_ALERT, \
    CLOSE_JIRA_TICKET, JIRA_ISSUE_NEW_STATE, DATE_FORMAT
from cloud_governance.common.elasticsearch.elasticsearch_operations import ElasticSearchOperations
from cloud_governance.common.jira.jira_operations import JiraOperations
from cloud_governance.common.logger.init_logger import logger
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp
from cloud_governance.common.mails.mail_message import MailMessage
from cloud_governance.common.mails.postfix import Postfix
from cloud_governance.main.environment_variables import environment_variables


class AbstractMonitorTickets(ABC):
    """
    This Abstract class perform the operations for monitoring tickets
    """

    def __init__(self):
        super().__init__()
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.__es_operations = ElasticSearchOperations()
        self.__es_index_cro = self.__environment_variables_dict.get('CRO_ES_INDEX', '')
        self.__account_name = self.__environment_variables_dict.get('account')
        self.__cloud_name = self.__environment_variables_dict.get('PUBLIC_CLOUD_NAME')
        self.__ticket_over_usage_limit = self.__environment_variables_dict.get('TICKET_OVER_USAGE_LIMIT')
        self.__default_admins = self.__environment_variables_dict.get('CRO_DEFAULT_ADMINS', [])
        self.__elasticsearch_queries = ElasticSearchQueries()
        self.__jira_operations = JiraOperations()
        self.__mail_message = MailMessage()
        self.__postfix = Postfix()

    def __get_all_in_progress_tickets(self, account_name: str = '', cloud_name: str = '', fields: list = None):
        """
        This method returns all in-progress tickets
        :param account_name:
        :param cloud_name:
        :param fields:
        :return:
        """
        account_name = account_name if account_name else self.__account_name
        cloud_name = cloud_name if cloud_name else self.__cloud_name
        match_conditions = [
            {"term": {"account_name.keyword": account_name}},
            {"term": {"cloud_name.keyword": cloud_name}}
        ]
        in_progress_tickets_query = self.__elasticsearch_queries.get_all_in_progress_tickets(
            match_conditions=match_conditions, fields=fields)
        in_progress_tickets_list = self.__es_operations.fetch_data_by_es_query(query=in_progress_tickets_query,
                                                                               es_index=self.__es_index_cro, filter_path='hits.hits._source')
        return in_progress_tickets_list

    @abstractmethod
    def update_budget_tag_to_resources(self, region_name: str, ticket_id: str, updated_budget: int):
        """
        This method updates the budget to cloud resources
        :param region_name:
        :param ticket_id:
        :param updated_budget:
        :return:
        """
        raise NotImplemented("This method is not implemented")

    @abstractmethod
    def update_duration_tag_to_resources(self, region_name: str, ticket_id: str, updated_duration: int):
        """
        This method updates the budget to cloud resources
        :param region_name:
        :param ticket_id:
        :param updated_duration:
        :return:
        """
        raise NotImplemented("This method is not implemented")

    @logger_time_stamp
    def extend_tickets_budget(self, ticket_id: str, region_name: str):
        """
        This method extends the ticket budget if any
        :param ticket_id:
        :param region_name:
        :return:
        """
        ticket_extended = False
        sub_ticket_ids = self.__jira_operations.get_budget_extend_tickets(ticket_id=ticket_id, ticket_state='inprogress')
        if sub_ticket_ids:
            total_budget_to_extend = self.__jira_operations.get_total_extend_budget(sub_ticket_ids=sub_ticket_ids)
            if string_equal_ignore_case(self.__cloud_name, 'AWS'):
                self.update_budget_tag_to_resources(region_name=region_name, ticket_id=ticket_id,
                                                    updated_budget=total_budget_to_extend)
                for sub_ticket_id in sub_ticket_ids:
                    self.__jira_operations.move_issue_state(ticket_id=sub_ticket_id, state='closed')
                logger.info(f'Updated the budget of the ticket: {ticket_id}')
                ticket_extended = True
        else:
            logger.info(f'No extended tickets for the TicketId: {ticket_id}')
        return ticket_extended

    @typeguard.typechecked
    @logger_time_stamp
    def extend_ticket_duration(self, ticket_id: str, region_name: str):
        """
        This method extends the duration of the ticket if any
        :param ticket_id:
        :param region_name:
        :return:
        """
        tickets_found = False
        sub_ticket_ids = self.__jira_operations.get_duration_extend_tickets(ticket_id=ticket_id, ticket_state='new')
        if sub_ticket_ids:
            total_duration_to_extend = self.__jira_operations.get_total_extend_duration(sub_ticket_ids=sub_ticket_ids)
            if string_equal_ignore_case(self.__cloud_name, 'AWS'):
                self.update_duration_tag_to_resources(region_name=region_name, ticket_id=ticket_id,
                                                      updated_duration=total_duration_to_extend)
                for sub_ticket_id in sub_ticket_ids:
                    self.__jira_operations.move_issue_state(ticket_id=sub_ticket_id, state='closed')
                logger.info(f'Updated the Duration of the ticket: {ticket_id}')
            tickets_found = True
        else:
            logger.info(f'No extended tickets for the TicketId: {ticket_id}')
        return tickets_found

    @typeguard.typechecked
    @logger_time_stamp
    def __close_and_update_ticket_data_in_es(self, ticket_id: str):
        """
        This method close the ticket and update in ElasticSearch
        :return:
        """
        data = {'timestamp': datetime.utcnow(), 'ticket_id_state': 'closed'}
        if self.__es_operations.check_elastic_search_connection():
            self.__es_operations.update_elasticsearch_index(index=self.__es_index_cro, id=ticket_id, metadata=data)
            self.__jira_operations.move_issue_state(ticket_id, state='CLOSED')

    @typeguard.typechecked
    @logger_time_stamp
    def _monitor_ticket_duration(self, ticket_id: str, region_name: str, duration: int, completed_duration: int, **kwargs):
        """
        This method monitors the ticket duration
        :param ticket_id:
        :param region_name:
        :param duration:
        :return:
        """
        user, cc = kwargs.get('user_cro'), self.__default_admins
        cc.append(kwargs.get('approved_manager'))
        remaining_duration = duration - completed_duration
        ticket_extended = False
        subject = body = None
        if remaining_duration <= FIRST_CRO_ALERT:
            ticket_extended = self.extend_ticket_duration(ticket_id=ticket_id, region_name=region_name)
            if not ticket_extended and remaining_duration == FIRST_CRO_ALERT:
                subject, body = self.__mail_message.cro_monitor_alert_message(user=user, days=FIRST_CRO_ALERT, ticket_id=ticket_id)
        elif remaining_duration == SECOND_CRO_ALERT:
            subject, body = self.__mail_message.cro_monitor_alert_message(user=user, days=SECOND_CRO_ALERT, ticket_id=ticket_id)
        else:
            if not ticket_extended:
                if remaining_duration <= CLOSE_JIRA_TICKET:
                    self.__close_and_update_ticket_data_in_es(ticket_id=ticket_id)
                    subject, body = self.__mail_message.cro_send_closed_alert(user, ticket_id)
        if subject and body:
            self.__postfix.send_email_postfix(to=user, cc=cc, subject=subject, content=body, mime_type='html',
                                              message_type='duration_exceeded_alert')

    @typeguard.typechecked
    @logger_time_stamp
    def _monitor_ticket_budget(self, ticket_id: str, region_name: str, budget: int, used_budget: int, **kwargs):
        """
        This method monitors the ticket budget
        :param ticket_id:
        :param region_name:
        :param budget:
        :param used_budget
        :return:
        """
        user, cc = kwargs.get('user_cro'), self.__default_admins
        remaining_budget = budget - used_budget
        threshold_budget = budget - (budget * (self.__ticket_over_usage_limit / 100))
        subject = body = None
        if threshold_budget >= remaining_budget >= 0:
            ticket_extended = self.extend_tickets_budget(ticket_id=ticket_id, region_name=region_name)
            if not ticket_extended:
                subject, body = self.__mail_message.cro_monitor_budget_remain_alert(user=user, budget=budget,
                                                                                    ticket_id=ticket_id,
                                                                                    used_budget=used_budget,
                                                                                    remain_budget=remaining_budget)
        elif remaining_budget <= 0:
            subject, body = self.__mail_message.cro_monitor_budget_remain_high_alert(user=user, budget=budget,
                                                                                     ticket_id=ticket_id,
                                                                                     used_budget=used_budget,
                                                                                     remain_budget=remaining_budget)
        if subject and body:
            self.__postfix.send_email_postfix(to=user, cc=cc, subject=subject, content=body, mime_type='html',
                                              message_type='budget_exceed_alert')

    @logger_time_stamp
    def _monitor_in_progress_tickets(self):
        """
        This method monitors in-progress tickets
        :return:
        """
        in_progress_tickets_list = self.__get_all_in_progress_tickets()
        for ticket_data in in_progress_tickets_list:
            source_data = ticket_data.get('_source')
            if source_data:
                ticket_id = source_data.get('ticket_id')
                region_name = source_data.get('region_name', '')
                budget = int(source_data.get('estimated_cost', 0))
                duration = int(source_data.get('duration', 0))
                used_budget = int(source_data.get('actual_cost', 0))
                ticket_start_date = datetime.strptime(source_data.get('ticket_opened_date'), DATE_FORMAT).date()
                completed_duration = (datetime.utcnow().date() - ticket_start_date).days
                self._monitor_ticket_budget(ticket_id=ticket_id, region_name=region_name, budget=budget,
                                            used_budget=used_budget,
                                            user_cro=source_data.get('user_cro'),
                                            approved_manager=source_data.get('approved_manager'))
                self._monitor_ticket_duration(ticket_id=ticket_id, region_name=region_name, duration=duration,
                                              completed_duration=completed_duration,
                                              user_cro=source_data.get('user_cro'),
                                              approved_manager=source_data.get('approved_manager')
                                              )

    def monitor_tickets(self):
        """
        This method monitor all tickets by status
        :return:
        """
        self._monitor_in_progress_tickets()



import json
import logging
import os
import tempfile
from datetime import date, datetime, timedelta

import typeguard
from botocore.exceptions import ClientError

from cloud_governance.common.clouds.aws.ec2.ec2_operations import EC2Operations
from cloud_governance.common.clouds.aws.s3.s3_operations import S3Operations
from cloud_governance.common.jira.jira import logger
from cloud_governance.common.logger.init_logger import handler
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp
from cloud_governance.common.mails.mail_message import MailMessage
from cloud_governance.common.mails.postfix import Postfix
from cloud_governance.main.environment_variables import environment_variables


class SendAggregatedAlerts:
    """
    This class send alerts to users which conditions are not satisfied by the policies
    """

    FILE_NAME = 'resources.json'
    GLOBAL_REGION = 'us-east-1'
    TODAY_DATE = str(date.today()).replace('-', '/')

    def __init__(self):
        self.__environment_variables = environment_variables.environment_variables_dict
        self.__bucket_name = self.__environment_variables.get('BUCKET_NAME')
        self.__bucket_key = self.__environment_variables.get('BUCKET_KEY')
        self.__policies = self.__environment_variables.get('POLICIES_TO_ALERT')
        self.__s3_operations = S3Operations(region_name='us-east-2', bucket=self.__bucket_name, logs_bucket_key=self.__bucket_key)
        self.__active_regions = EC2Operations().get_active_regions()
        self.__kerberos_users = self.__get_kerberos_users_for_iam_users()
        self.__global_region_policies = ['s3-inactive', 'empty-roles']
        self.__mail_alert_days = self.__environment_variables.get('MAIL_ALERT_DAYS')
        self.__policy_action_days = self.__environment_variables.get('POLICY_ACTIONS_DAYS')
        self.__mail_message = MailMessage()
        self.__postfix = Postfix()

    @logger_time_stamp
    def __get_kerberos_users_for_iam_users(self):
        """
        This method returns the users which IAM users are not kerberos username
        :return:
        """
        responses = {}
        users = self.__environment_variables.get('KERBEROS_USERS')
        for iam_user, kerberos_user in users.items():
            responses[iam_user.lower()] = kerberos_user.lower()
        return responses

    def __get_users_agg_result(self, policy_result: list, agg_users_result: dict, policy_name: str, region: str):
        """
        This method returns the aggregated users resources list
        :param agg_users_result:
        :param policy_result:
        :return:
        """
        if policy_result:
            for response in policy_result:
                if type(response) == dict:
                    skip_policy = response.get('Skip')
                    if skip_policy in ('NA', '', None):
                        user = response.pop('User').lower()
                        response['Region'] = region
                        response['Policy'] = policy_name
                        if user in self.__kerberos_users.keys():
                            user = self.__kerberos_users.get(user)
                        agg_users_result.setdefault(user, []).append(response)

    def __get_policy_data_in_bucket(self, region: str, policy: str):
        """
        This method returns the policy data in s3 bucket
        :param region:
        :param policy:
        :return:
        """
        try:
            policy_save_path = f'{self.__bucket_key}/{region}/{policy}'
            bucket_path_file = self.__s3_operations.get_last_objects(bucket=self.__bucket_name, key_prefix=f'{policy_save_path}/{self.TODAY_DATE}')
            policy_s3_response = self.__s3_operations.get_last_s3_policy_content(s3_file_path=bucket_path_file, file_name=self.FILE_NAME)
            return json.loads(policy_s3_response) if policy_s3_response else []
        except ClientError as err:
            logger.info(err)
            return []

    @logger_time_stamp
    def __get_policy_users_list(self):
        """
        This method gets the latest policy responses
        :return:
        """
        agg_users_result = {}
        for policy in self.__policies:
            run_global_region = True if policy in self.__global_region_policies else False
            for region in self.__active_regions:
                if (region == self.GLOBAL_REGION and run_global_region) or not run_global_region:
                    self.__get_users_agg_result(policy_result=self.__get_policy_data_in_bucket(region=region, policy=policy),
                                                agg_users_result=agg_users_result, policy_name=policy, region=region)
                if region == self.GLOBAL_REGION and run_global_region:
                    break
        return agg_users_result

    def __get_policy_agg_data_by_region(self, policy_data: dict):
        """
        This method returns the policy data agg by region
        :param policy_data:
        :return:
        """
        agg_policy_region_result = {}
        for policy_name, policy_region_data in policy_data.items():
            agg_policy_region_result[policy_name] = {}
            for region_data in policy_region_data:
                region_name = region_data.get('Region').lower()
                agg_policy_region_result[policy_name].setdefault(region_name, []).append(region_data)
        return agg_policy_region_result

    @logger_time_stamp
    def __get_policy_agg_data(self, user_policy_data: list):
        """
        This method returns the data agg by policy
        :param user_policy_data:
        :return:
        """
        agg_policy_result = {}
        for result in user_policy_data:
            policy_name = result.get('Policy').lower()
            days = int(result.get('Days', 0))
            if days in self.__mail_alert_days or days in self.__policy_action_days:
                result['Action'] = 'Deleted' if days in self.__policy_action_days else 'Monitoring'
                result['DeletedDay'] = (datetime.now() + timedelta(days=self.__policy_action_days[0] - days)).date()
                agg_policy_result.setdefault(policy_name, []).append(result)
        return self.__get_policy_agg_data_by_region(policy_data=agg_policy_result)

    @logger_time_stamp
    def __send_mail_alerts_to_users(self):
        """
        This method send mail alerts to users
        :return:
        """
        policy_agg_users_list = self.__get_policy_users_list()
        for user, user_policy_data in policy_agg_users_list.items():
            handler.setLevel(logging.WARN)
            agg_policy_data = self.__get_policy_agg_data(user_policy_data=user_policy_data)
            if agg_policy_data:
                handler.setLevel(logging.INFO)
                subject, body = self.__mail_message.get_agg_policies_mail_message(user=user, user_resources=agg_policy_data)
                self.__postfix.send_email_postfix(subject=subject, content=body, to=user, cc=[], mime_type='html')

    @logger_time_stamp
    def run(self):
        """
        This method start the other methods
        :return:
        """
        self.__send_mail_alerts_to_users()



from azure.identity import DefaultAzureCredential
from azure.mgmt.billing import BillingManagementClient
from azure.mgmt.costmanagement import CostManagementClient
from azure.mgmt.subscription import SubscriptionClient

from cloud_governance.main.environment_variables import environment_variables


class AzureOperations:
    """
    This class represents the Azure account operations
    """

    def __init__(self):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.__tenant_id = self.__environment_variables_dict.get('AZURE_TENANT_ID')
        self.__client_id = self.__environment_variables_dict.get('AZURE_CLIENT_ID')
        self.__client_secret = self.__environment_variables_dict.get('AZURE_CLIENT_SECRET')
        self.__default_creds = DefaultAzureCredential()
        self.__subscription_client = SubscriptionClient(credential=self.__default_creds)
        self.cost_mgmt_client = CostManagementClient(credential=self.__default_creds)
        self.subscription_id, self.account_name = self.__get_subscription_id()
        self.billing_client = BillingManagementClient(credential=self.__default_creds, subscription_id=self.subscription_id)
        self.__account_id = self.__environment_variables_dict.get('AZURE_ACCOUNT_ID')
        self.cloud_name = 'AZURE'
        self.scope = f'subscriptions/{self.subscription_id}'

    def __get_subscription_id(self):
        """
        This methods return the subscription ID
        @return:
        """
        subscription_list = self.__subscription_client.subscriptions.list()
        for subscription in subscription_list:
            data_dict = subscription.as_dict()
            subscription_id = data_dict.get('subscription_id')
            account_name = data_dict.get('display_name').split()[0]
            return subscription_id, account_name
        return '', ''

    def get_billing_profiles(self):
        """
        This method get the billing profiles from the azure account
        """
        response = self.billing_client.billing_profiles.list_by_billing_account(self.__account_id)
        return response








AWS: str = "AWS"
GCP: str = "GCP"
AZURE: str = "AZURE"
IBM: str = "IBM"
JIRA_ISSUE_NEW_STATE: str = 'New'
JIRA_ISSUE_REFINEMENT_STATE: str = 'Refinement'
JIRA_ISSUE_CLOSED_STATE: str = 'Closed'
JIRA_ISSUE_IN_PROGRESS_STATE: str = 'In Progress'
FIRST_CRO_ALERT: int = 5
SECOND_CRO_ALERT: int = 3
CLOSE_JIRA_TICKET: int = 0
DEFAULT_ROUND_DIGITS: int = 3
DATE_FORMAT: str = '%Y-%m-%d'



from datetime import datetime

import boto3
from dateutil.relativedelta import relativedelta


class SavingsPlansOperations:
    """
    This class is for implementing savings plans methods
    """

    SAVINGS_PLANS = 'savingsPlans'
    NEXT_TOKEN = 'nextToken'
    DATE_TIME_FORMAT = '%Y-%m-%dT%H:%M:%S.%fZ'
    DEFAULT_ROUND_DIGITS = 3

    def __init__(self, savings_plan_client=None):
        self.savings_plan_client = savings_plan_client if savings_plan_client else boto3.client(self.SAVINGS_PLANS.lower())

    def get_savings_plans_months(self, start_date: datetime, end_date: datetime):
        """
        This method returns the savings plans months ranges
        :param start_date:
        :param end_date:
        :return:
        """
        start_end_dates = []
        month_start_date = start_date
        while month_start_date <= end_date:
            # Calculate the month end date as the last day of the current month
            month_end_date = min(
                datetime(month_start_date.year, month_start_date.month, 1) + relativedelta(months=1) - relativedelta(
                    days=1), end_date)
            start_end_dates.append({'start': month_start_date, 'end': month_end_date, 'month': month_start_date.strftime("%b")})
            month_start_date += relativedelta(months=1)
            month_start_date = datetime(month_start_date.year, month_start_date.month, 1)
        return start_end_dates

    def get_savings_filter_data(self, savings_plans_list: list = None):
        """
        This method returns the savings plans filter values
        :return:
        """
        responses = []
        if not savings_plans_list:
            savings_plans_list = self.get_savings_plans_list()
        extract_values = ['savingsPlanId', 'state', 'savingsPlanType',
                          'paymentOption', 'productTypes', 'commitment', 'upfrontPaymentAmount',
                          'recurringPaymentAmount']
        for savings_plans in savings_plans_list:
            start_date = datetime.strptime(savings_plans.get('start'), self.DATE_TIME_FORMAT)
            end_date = datetime.strptime(savings_plans.get('end'), self.DATE_TIME_FORMAT)
            total_payment = savings_plans.get('upfrontPaymentAmount')
            daily_payment = float(total_payment) / (24 * 60)
            for date_range in self.get_savings_plans_months(start_date=start_date, end_date=end_date):
                start, end, month = date_range.get('start'), date_range.get('end'), date_range.get('month')
                savings_id = savings_plans.get('savingsPlanId')
                days = (end - start).days + 1
                month_payment = round(days * daily_payment, self.DEFAULT_ROUND_DIGITS)
                start, end = str(start.date()), str(end.date())
                monthly_savings_data = {'start': start, 'end': end, 'filter_date': f"{start}-{month}", 'index_id': f"{start}-{savings_id}", "CloudName": "AWS", 'SavingsMonthlyPayment': month_payment, 'CostType': 'savings_plans'}
                monthly_savings_data.update({value.title(): savings_plans.get(value) for value in extract_values})
                responses.append(monthly_savings_data)
        return responses

    def get_savings_plans_list(self, states: list = [], **kwargs):
        """
        This method returns the savings plans list
        :param states:
        :return:
        """
        results = {}
        kwargs.update({'states': states})
        if not kwargs.get('states'):
            kwargs.pop('states')
        response = self.savings_plan_client.describe_savings_plans(**kwargs)
        results[self.SAVINGS_PLANS] = response.get(self.SAVINGS_PLANS)
        while response.get(self.NEXT_TOKEN):
            response = self.savings_plan_client.describe_savings_plans(**kwargs)
            results[self.SAVINGS_PLANS].append(response.get(self.SAVINGS_PLANS))
        return results[self.SAVINGS_PLANS]



from tempfile import TemporaryDirectory

from cloud_governance.cloud_resource_orchestration.monitor.cloud_monitor import CloudMonitor
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp
from cloud_governance.main.environment_variables import environment_variables


@logger_time_stamp
def run_cloud_management():
    """This method run the cloud management"""
    environment_variables_dict = environment_variables.environment_variables_dict
    with TemporaryDirectory() as cache_temp_dir:
        environment_variables_dict['TEMPORARY_DIRECTORY'] = cache_temp_dir
        environment_variables_dict['policy'] = 'cloud_resource_orchestration'
        return CloudMonitor().run()


def run_cloud_resource_orchestration():
    run_cloud_management()



from datetime import datetime, timedelta

from google.cloud import bigquery
from typeguard import typechecked

from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp
from cloud_governance.main.environment_variables import environment_variables

import google.auth
from googleapiclient.discovery import build
from googleapiclient.errors import HttpError


class GoogleAccount:
    """
    This class is for Google account operations
    """

    RETRIES = 3

    def __init__(self):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.__client = None
        if self.__environment_variables_dict.get('GOOGLE_APPLICATION_CREDENTIALS'):
            self.__creds, _ = google.auth.default()
            self.__client = bigquery.Client()

    @typechecked()
    @logger_time_stamp
    def get_dates(self, diff_days: int):
        """
        This method returns the start and end date
        :param diff_days:
        :return:
        """
        end_date = datetime.now()
        start_date = end_date - timedelta(days=diff_days)
        return start_date, end_date

    @typechecked()
    @logger_time_stamp
    def query_list(self, queries: list):
        """
        This method returns the query results that scans from the BigQuery
        :param queries:
        :return:
        """
        queries_results = []
        for idx, query in enumerate(queries):
            parent_job = self.__client.query(query)
            results = parent_job.result()  # Waits for job to complete.
            query_rows = []
            for row in results:
                query_rows.append(dict(row))
            queries_results.append(query_rows)
        return queries_results



from typing import Callable

import typeguard

from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp
from cloud_governance.main.environment_variables import environment_variables


class Utils:
    """
    This is global methods
    """

    def __init__(self, region: str = 'us-east-2'):
        self.region = region
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.__update_tag_bulks = self.__environment_variables_dict.get('UPDATE_TAG_BULKS')

    @typeguard.typechecked
    def get_details_resource_list(self, func_name: Callable, input_tag: str, check_tag: str, **kwargs):
        """
        This method fetch all Items of the resource i.e: EC2, IAM
        :param func_name:
        :param input_tag:
        :param check_tag:
        :return:
        """
        resource_list = []
        resources = func_name(**kwargs)
        resource_list.extend(resources[input_tag])
        while check_tag in resources.keys():
            if check_tag == 'NextToken':
                resources = func_name(NextToken=resources[check_tag], **kwargs)
            elif check_tag == 'Marker':
                resources = func_name(Marker=resources[check_tag], **kwargs)
            resource_list.extend(resources[input_tag])
        return resource_list

    @logger_time_stamp
    def __tag_resources(self, client_method: Callable, resource_ids: list, tags: list, tags_name: str = 'Tags'):
        """
        This method tag resources
        :param client_method:
        :param resource_ids:
        :param tags:
        :param tags_name:
        :return:
        """
        if tags_name == 'Tags':
            client_method(Resources=resource_ids, Tags=tags)

    @logger_time_stamp
    def __split_run_bulks(self, iterable: list, limit: int = 1):
        """
        This method splits run into bulk depends on threads limit
        @return: run bulks
        """
        result = []
        length = len(iterable)
        for ndx in range(0, length, limit):
            result.append(iterable[ndx:min(ndx + limit, length)])
        return result

    @typeguard.typechecked
    @logger_time_stamp
    def tag_aws_resources(self, client_method: Callable, tags: list, resource_ids: list):
        """
        This method tag the aws resources with batch wise of 20
        :param client_method:
        :param tags:
        :param resource_ids:
        :return:
        """
        if tags:
            bulk_resource_ids_list = self.__split_run_bulks(iterable=resource_ids, limit=self.__update_tag_bulks)  # split the aws resource_ids into batches
            co = 0
            cpu_based_resource_ids_list = self.__split_run_bulks(iterable=bulk_resource_ids_list, limit=self.__update_tag_bulks)
            for cpu_based_resource_ids_list in cpu_based_resource_ids_list:
                for resource_ids_list in cpu_based_resource_ids_list:
                    self.__tag_resources(client_method, resource_ids_list, tags)
                    co += 1
            return co






def string_equal_ignore_case(value1: str, value2: str, *args) -> bool:
    """
    This method finds all values are equal and returns bool
    :return:
    """
    equal = value1.lower() == value2.lower()
    if args:
        for val in args:
            equal = value1.lower() == val.lower() and equal
    return equal


def integer_equal(value1: int, value2: int, *args) -> bool:
    """
    This method finds all values are equal and returns bool
    :param value1:
    :param value2:
    :param args:
    :return:
    """
    equal = value1 == value2
    if args:
        for val in args:
            equal = equal and value1 == val
    return equal


def get_tag_value_by_name(tags: list, tag_name: str) -> str:
    """
    This method returns the tag_value
    :param tags:
    :param tag_name:
    :return:
    """
    for tag in tags:
        key = tag.get('Key')
        value = tag.get('Value')
        if string_equal_ignore_case(key, tag_name):
            return value
    return ''



import argparse
import os

import tempfile
from ast import literal_eval

import boto3

from cloud_governance.main.environment_variables_exceptions import ParseFailed


class EnvironmentVariables:
    """
    This class manages the environment variable parameters
    """

    def __init__(self):
        self._environment_variables_dict = {}

        # env files override true ENV. Not best order, but easier to write :/
        # .env.generated can be auto-generated (by an external tool) based on the local cluster's configuration.
        for env in ".env", ".env.generated":
            try:
                with open(env) as f:
                    for line in f.readlines():
                        key, found, value = line.strip().partition("=")
                        if not found:
                            print("ERROR: invalid line in {env}: {line.strip()}")
                            continue
                        if key in os.environ:
                            continue  # prefer env to env file
                        os.environ[key] = value

            except FileNotFoundError:
                pass  # ignore

        ##################################################################################################
        # dynamic parameters - configure for local run
        # parameters for running policies
        self._environment_variables_dict['account'] = EnvironmentVariables.get_env('account', '').upper().strip()
        self._environment_variables_dict['AWS_DEFAULT_REGION'] = EnvironmentVariables.get_env('AWS_DEFAULT_REGION', '')
        self._environment_variables_dict['log_level'] = EnvironmentVariables.get_env('log_level', 'INFO')
        self._environment_variables_dict['PRINT_LOGS'] = EnvironmentVariables.get_boolean_from_environment('PRINT_LOGS', True)
        if not self._environment_variables_dict['AWS_DEFAULT_REGION']:
            self._environment_variables_dict['AWS_DEFAULT_REGION'] = 'us-east-2'
        self._environment_variables_dict['PUBLIC_CLOUD_NAME'] = EnvironmentVariables.get_env('PUBLIC_CLOUD_NAME', 'AWS')
        if EnvironmentVariables.get_env('AWS_ACCESS_KEY_ID', '') and EnvironmentVariables.get_env('AWS_SECRET_ACCESS_KEY', ''):
            self._environment_variables_dict['account'] = self.get_aws_account_alias_name().upper().replace('OPENSHIFT-', '')
        self._environment_variables_dict['policy'] = EnvironmentVariables.get_env('policy', '')

        self._environment_variables_dict['aws_non_cluster_policies'] = ['ec2_idle', 'ec2_stop', 'ec2_run', 'ebs_in_use',
                                                                        'ebs_unattached', 's3_inactive',
                                                                        'empty_roles', 'ip_unattached',
                                                                        'unused_nat_gateway',
                                                                        'zombie_snapshots', 'skipped_resources',
                                                                        'monthly_report']
        es_index = 'cloud-governance-policy-es-index'
        self._environment_variables_dict['cost_policies'] = ['cost_explorer', 'cost_over_usage', 'cost_billing_reports',
                                                             'cost_explorer_payer_billings', 'spot_savings_analysis']
        self._environment_variables_dict['ibm_policies'] = ['tag_baremetal', 'tag_vm', 'ibm_cost_report',
                                                            'ibm_cost_over_usage']
        if self._environment_variables_dict['policy'] in self._environment_variables_dict['cost_policies']:
            es_index = 'cloud-governance-global-cost-billing-index'

        # AWS env vars
        self._environment_variables_dict['resource_name'] = EnvironmentVariables.get_env('resource_name', '')
        self._environment_variables_dict['mandatory_tags'] = EnvironmentVariables.get_env('mandatory_tags', '{}')
        self._environment_variables_dict['tag_operation'] = EnvironmentVariables.get_env('tag_operation', 'read')
        self._environment_variables_dict['validate_type'] = EnvironmentVariables.get_env('validate_type', '')
        self._environment_variables_dict['user_tags'] = EnvironmentVariables.get_env('user_tags', '')
        self._environment_variables_dict['user_tag_operation'] = EnvironmentVariables.get_env('user_tag_operation', '')
        self._environment_variables_dict['username'] = EnvironmentVariables.get_env('username', '')
        self._environment_variables_dict['remove_tags'] = EnvironmentVariables.get_env('remove_tags', '')
        self._environment_variables_dict['resource'] = EnvironmentVariables.get_env('resource', '')
        self._environment_variables_dict['cluster_tag'] = EnvironmentVariables.get_env('cluster_tag', '')
        self._environment_variables_dict['service_type'] = EnvironmentVariables.get_env('service_type', '')
        self._environment_variables_dict['TABLE_NAME'] = EnvironmentVariables.get_env('TABLE_NAME', '')
        self._environment_variables_dict['REPLACE_ACCOUNT_NAME'] = EnvironmentVariables.get_env('REPLACE_ACCOUNT_NAME', '{}')
        self._environment_variables_dict['DAYS_TO_DELETE_RESOURCE'] = int(EnvironmentVariables.get_env('DAYS_TO_DELETE_RESOURCE', '7'))

        # AWS Cost Explorer tags
        self._environment_variables_dict['cost_metric'] = EnvironmentVariables.get_env('cost_metric', 'UnblendedCost')
        self._environment_variables_dict['start_date'] = EnvironmentVariables.get_env('start_date', '')
        self._environment_variables_dict['end_date'] = EnvironmentVariables.get_env('end_date', '')
        self._environment_variables_dict['granularity'] = EnvironmentVariables.get_env('granularity', 'DAILY')
        self._environment_variables_dict['cost_explorer_tags'] = EnvironmentVariables.get_env('cost_explorer_tags', '{}')
        self._environment_variables_dict['PUBLIC_CLOUD_NAME'] = EnvironmentVariables.get_env('PUBLIC_CLOUD_NAME', 'AWS')

        # AZURE Credentials
        self._environment_variables_dict['AZURE_ACCOUNT_ID'] = EnvironmentVariables.get_env('AZURE_ACCOUNT_ID', '')
        self._environment_variables_dict['AZURE_CLIENT_ID'] = EnvironmentVariables.get_env('AZURE_CLIENT_ID', '')
        self._environment_variables_dict['AZURE_TENANT_ID'] = EnvironmentVariables.get_env('AZURE_TENANT_ID', '')
        self._environment_variables_dict['AZURE_CLIENT_SECRET'] = EnvironmentVariables.get_env('AZURE_CLIENT_SECRET', '')
        if self._environment_variables_dict['AZURE_CLIENT_ID'] and self._environment_variables_dict['AZURE_TENANT_ID']\
                and self._environment_variables_dict['AZURE_CLIENT_SECRET']:
            self._environment_variables_dict['PUBLIC_CLOUD_NAME'] = 'AZURE'
        self._environment_variables_dict['TOTAL_ACCOUNTS'] = EnvironmentVariables.get_boolean_from_environment('TOTAL_ACCOUNTS', False)

        # IBM env vars
        self._environment_variables_dict['IBM_ACCOUNT_ID'] = EnvironmentVariables.get_env('IBM_ACCOUNT_ID', '')
        self._environment_variables_dict['IBM_API_USERNAME'] = EnvironmentVariables.get_env('IBM_API_USERNAME', '')
        self._environment_variables_dict['IBM_API_KEY'] = EnvironmentVariables.get_env('IBM_API_KEY', '')
        self._environment_variables_dict['USAGE_REPORTS_APIKEY'] = EnvironmentVariables.get_env('USAGE_REPORTS_APIKEY', '')
        if self._environment_variables_dict['USAGE_REPORTS_APIKEY']:
            self._environment_variables_dict['PUBLIC_CLOUD_NAME'] = 'IBM'
        self._environment_variables_dict['month'] = EnvironmentVariables.get_env('month', '')
        self._environment_variables_dict['year'] = EnvironmentVariables.get_env('year', '')
        self._environment_variables_dict['COST_CENTER_OWNER'] = EnvironmentVariables.get_env('COST_CENTER_OWNER', '')

        self._environment_variables_dict['tag_remove_name'] = EnvironmentVariables.get_env('tag_remove_name', '')
        self._environment_variables_dict['tag_custom'] = EnvironmentVariables.get_env('tag_custom', '{}')

        # Common env vars
        self._environment_variables_dict['dry_run'] = EnvironmentVariables.get_env('dry_run', 'yes')
        self._environment_variables_dict['FORCE_DELETE'] = EnvironmentVariables.get_boolean_from_environment('FORCE_DELETE', False)
        self._environment_variables_dict['policy_output'] = EnvironmentVariables.get_env('policy_output', '')
        self._environment_variables_dict['bucket'] = EnvironmentVariables.get_env('bucket', '')
        self._environment_variables_dict['file_path'] = EnvironmentVariables.get_env('file_path', '')
        self._environment_variables_dict['file_name'] = EnvironmentVariables.get_env('file_name', '')

        # common elastic search vars
        self._environment_variables_dict['upload_data_elk'] = EnvironmentVariables.get_env('upload_data_elk', '')
        self._environment_variables_dict['upload_data_es'] = EnvironmentVariables.get_env('upload_data_es', '')
        self._environment_variables_dict['es_host'] = EnvironmentVariables.get_env('es_host', '')
        self._environment_variables_dict['es_port'] = EnvironmentVariables.get_env('es_port', '')
        self._environment_variables_dict['es_index'] = EnvironmentVariables.get_env('es_index', es_index)
        self._environment_variables_dict['es_doc_type'] = EnvironmentVariables.get_env('es_doc_type', '')
        self._environment_variables_dict['ES_TIMEOUT'] = EnvironmentVariables.get_env('ES_TIMEOUT', 2000)

        # GitHub credentials
        self._environment_variables_dict['git_access_token'] = EnvironmentVariables.get_env('git_access_token', '')
        self._environment_variables_dict['git_repo'] = EnvironmentVariables.get_env('git_repo', '')
        self._environment_variables_dict['several_repos'] = EnvironmentVariables.get_env('several_repos', '')

        # Mail alerts env vars
        # ldap env var
        self._environment_variables_dict['LDAP_HOST_NAME'] = EnvironmentVariables.get_env('LDAP_HOST_NAME', '')
        self._environment_variables_dict['SENDER_MAIL'] = EnvironmentVariables.get_env('SENDER_MAIL', '')
        self._environment_variables_dict['SENDER_PASSWORD'] = EnvironmentVariables.get_env('SENDER_PASSWORD', '')
        self._environment_variables_dict['REPLY_TO'] = EnvironmentVariables.get_env('REPLY_TO', 'dev-null@redhat.com')
        self._environment_variables_dict['special_user_mails'] = EnvironmentVariables.get_env('special_user_mails', '{}')
        self._environment_variables_dict['account_admin'] = EnvironmentVariables.get_env('account_admin', '')
        self._environment_variables_dict['IGNORE_MAILS'] = EnvironmentVariables.get_env('IGNORE_MAILS', '')
        self._environment_variables_dict['MAXIMUM_THRESHOLD'] = EnvironmentVariables.get_env('MAXIMUM_THRESHOLD', '')
        self._environment_variables_dict['to_mail'] = EnvironmentVariables.get_env('to_mail', '[]')
        self._environment_variables_dict['cc_mail'] = EnvironmentVariables.get_env('cc_mail', '[]')

        # Google Drive env vars
        self._environment_variables_dict['GOOGLE_APPLICATION_CREDENTIALS'] = EnvironmentVariables.get_env('GOOGLE_APPLICATION_CREDENTIALS', '')
        self._environment_variables_dict['SPREADSHEET_ID'] = EnvironmentVariables.get_env('SPREADSHEET_ID', '')

        # AWS Top Acconut
        self._environment_variables_dict['AWS_ACCOUNT_ROLE'] = EnvironmentVariables.get_env('AWS_ACCOUNT_ROLE', '')
        self._environment_variables_dict['PAYER_SUPPORT_FEE_CREDIT'] = EnvironmentVariables.get_env('PAYER_SUPPORT_FEE_CREDIT', 0)
        self._environment_variables_dict['TEMPORARY_DIR'] = EnvironmentVariables.get_env('TEMPORARY_DIR', '/tmp')
        self._environment_variables_dict['COST_CENTER_OWNER'] = EnvironmentVariables.get_env('COST_CENTER_OWNER', '{}')

        # Jira env parameters
        self._environment_variables_dict['JIRA_URL'] = EnvironmentVariables.get_env('JIRA_URL', '')
        self._environment_variables_dict['JIRA_USERNAME'] = EnvironmentVariables.get_env('JIRA_USERNAME', '')
        self._environment_variables_dict['JIRA_TOKEN'] = EnvironmentVariables.get_env('JIRA_TOKEN', '')
        self._environment_variables_dict['JIRA_QUEUE'] = EnvironmentVariables.get_env('JIRA_QUEUE', '')
        self._environment_variables_dict['JIRA_PASSWORD'] = EnvironmentVariables.get_env('JIRA_PASSWORD', '')

        # Cloud Resource Orchestration
        self._environment_variables_dict['CRO_PORTAL'] = EnvironmentVariables.get_env('CRO_PORTAL', '')
        self._environment_variables_dict['CLOUD_NAME'] = EnvironmentVariables.get_env('CLOUD_NAME', '')
        self._environment_variables_dict['MONITOR'] = EnvironmentVariables.get_env('MONITOR', '')
        self._environment_variables_dict['MANAGEMENT'] = EnvironmentVariables.get_boolean_from_environment('MANAGEMENT', False)

        # GCP Account
        self._environment_variables_dict['GCP_DATABASE_NAME'] = EnvironmentVariables.get_env('GCP_DATABASE_NAME')
        self._environment_variables_dict['GCP_DATABASE_TABLE_NAME'] = EnvironmentVariables.get_env('GCP_DATABASE_TABLE_NAME')
        if self._environment_variables_dict.get('GCP_DATABASE_TABLE_NAME'):
            self._environment_variables_dict['PUBLIC_CLOUD_NAME'] = 'GCP'

        self._environment_variables_dict['EMAIL_ALERT'] = EnvironmentVariables.get_boolean_from_environment('EMAIL_ALERT', True)
        self._environment_variables_dict['MANAGER_EMAIL_ALERT'] = EnvironmentVariables.get_boolean_from_environment('MANAGER_EMAIL_ALERT', True)
        self._environment_variables_dict['UPDATE_TAG_BULKS'] = int(EnvironmentVariables.get_env('UPDATE_TAG_BULKS', '20'))

        # policies aggregate alert
        self._environment_variables_dict['BUCKET_NAME'] = EnvironmentVariables.get_env('BUCKET_NAME')
        self._environment_variables_dict['BUCKET_KEY'] = EnvironmentVariables.get_env('BUCKET_KEY')
        self._environment_variables_dict['MAIL_ALERT_DAYS'] = literal_eval(EnvironmentVariables.get_env('MAIL_ALERT_DAYS', '[]'))
        self._environment_variables_dict['POLICY_ACTIONS_DAYS'] = literal_eval(EnvironmentVariables.get_env('POLICY_ACTIONS_DAYS', '[]'))
        self._environment_variables_dict['DEFAULT_ADMINS'] = literal_eval(EnvironmentVariables.get_env('DEFAULT_ADMINS', '[]'))
        self._environment_variables_dict['KERBEROS_USERS'] = literal_eval(EnvironmentVariables.get_env('KERBEROS_USERS', '[]'))
        self._environment_variables_dict['POLICIES_TO_ALERT'] = literal_eval(EnvironmentVariables.get_env('POLICIES_TO_ALERT', '[]'))
        if self._environment_variables_dict.get('policy') in ['send_aggregated_alerts']:
            self._environment_variables_dict['COMMON_POLICIES'] = True
        # CRO -- Cloud Resource Orch
        self._environment_variables_dict['CLOUD_RESOURCE_ORCHESTRATION'] = EnvironmentVariables.get_boolean_from_environment('CLOUD_RESOURCE_ORCHESTRATION', False)
        self._environment_variables_dict['USER_COST_INDEX'] = EnvironmentVariables.get_env('USER_COST_INDEX', '')
        self._environment_variables_dict['CRO_ES_INDEX'] = EnvironmentVariables.get_env('CRO_ES_INDEX', 'cloud-governance-resource-orchestration')
        self._environment_variables_dict['CRO_COST_OVER_USAGE'] = int(EnvironmentVariables.get_env('CRO_COST_OVER_USAGE', '500'))
        self._environment_variables_dict['CRO_DEFAULT_ADMINS'] = literal_eval(EnvironmentVariables.get_env('CRO_DEFAULT_ADMINS', "[]"))
        self._environment_variables_dict['CRO_DURATION_DAYS'] = int(EnvironmentVariables.get_env('CRO_DURATION_DAYS', '30'))
        self._environment_variables_dict['RUN_ACTIVE_REGIONS'] = EnvironmentVariables.get_boolean_from_environment('RUN_ACTIVE_REGIONS', False)
        self._environment_variables_dict['CRO_RESOURCE_TAG_NAME'] = EnvironmentVariables.get_env('CRO_RESOURCE_TAG_NAME', 'TicketId')
        self._environment_variables_dict['CRO_REPLACED_USERNAMES'] = literal_eval(EnvironmentVariables.get_env('CRO_REPLACED_USERNAMES', "['osdCcsAdmin']"))
        self._environment_variables_dict['CE_PAYER_INDEX'] = EnvironmentVariables.get_env('CE_PAYER_INDEX', '')
        self._environment_variables_dict['EMAIL_TO'] = EnvironmentVariables.get_env('EMAIL_TO', '')
        self._environment_variables_dict['EMAIL_CC'] = literal_eval(EnvironmentVariables.get_env('EMAIL_CC', "[]"))
        self._environment_variables_dict['MANAGER_ESCALATION_DAYS'] = int(EnvironmentVariables.get_env('MANAGER_ESCALATION_DAYS', '3'))
        self._environment_variables_dict['GLOBAL_CLOUD_ADMIN'] = EnvironmentVariables.get_env('GLOBAL_CLOUD_ADMIN', 'natashba')
        self._environment_variables_dict['TICKET_OVER_USAGE_LIMIT'] = int(EnvironmentVariables.get_env('TICKET_OVER_USAGE_LIMIT', '80'))

        #  AWS Athena
        self._environment_variables_dict['S3_RESULTS_PATH'] = EnvironmentVariables.get_env('S3_RESULTS_PATH', '')
        self._environment_variables_dict['DEFAULT_ROUND_DIGITS'] = \
            int(EnvironmentVariables.get_env('DEFAULT_ROUND_DIGITS', '3'))
        self._environment_variables_dict['ATHENA_DATABASE_NAME'] = EnvironmentVariables.get_env('ATHENA_DATABASE_NAME', '')
        self._environment_variables_dict['ATHENA_TABLE_NAME'] = EnvironmentVariables.get_env('ATHENA_TABLE_NAME', '')




    @staticmethod
    def to_bool(arg, def_val: bool = None):
        if isinstance(arg, bool):
            return arg
        if isinstance(arg, (int, float)):
            return arg != 0
        if isinstance(arg, str):
            arg = arg.lower()
            if arg == 'true' or arg == 'yes':
                return True
            elif arg == 'false' or arg == 'no':
                return False
            try:
                arg1 = int(arg)
                return arg1 != 0
            except Exception:
                pass
        if def_val is not None:
            return def_val
        raise ParseFailed(f'Cannot parse {arg} as a boolean value')

    def get_aws_account_alias_name(self):
        """
        This method return the aws account alias name
        :return:
        """
        iam_client = boto3.client('iam')
        try:
            account_alias = iam_client.list_account_aliases()['AccountAliases']
            if account_alias:
                return account_alias[0].upper()
        except:
            return os.environ.get('account', '').upper()


    @staticmethod
    def get_env(var: str, defval=''):
        lcvar = var.lower()
        dashvar = lcvar.replace('_', '-')
        parser = argparse.ArgumentParser(description='Run CloudGovernance', allow_abbrev=False)
        if lcvar == dashvar:
            parser.add_argument(f"--{lcvar}", default=os.environ.get(var, defval), type=str, metavar='String', help=var)
        else:
            parser.add_argument(f"--{lcvar}", f"--{dashvar}", default=os.environ.get(var, defval), type=str,
                                metavar='String', help=var)
        args, ignore = parser.parse_known_args()
        if hasattr(args, lcvar):
            return getattr(args, lcvar)
        else:
            return os.environ.get(var, defval)

    @staticmethod
    def get_boolean_from_environment(var: str, defval: bool):
        return EnvironmentVariables.to_bool(EnvironmentVariables.get_env(var), defval)

    @property
    def environment_variables_dict(self):
        """
        This method is getter
        """
        return self._environment_variables_dict


environment_variables = EnvironmentVariables()
# env vars examples
# os.environ['AWS_DEFAULT_REGION'] = 'us-east-2'
# os.environ['AWS_DEFAULT_REGION'] = 'all'
# os.environ['policy'] = 'zombie_cluster_resource'
# os.environ['validate_type'] = 'tags'
# os.environ['user_tags'] = "['Budget', 'User', 'Owner', 'Manager', 'Environment', 'Project']"
# os.environ['cost_metric'] = ''
# os.environ['start_date'] = ''
# os.environ['end_date'] = ''
# os.environ['granularity'] = ''
# os.environ['policy'] = 'ec2_untag'
# os.environ['policy'] = 'zombie_cluster_resource'
# os.environ['dry_run'] = 'yes'
# os.environ['tag_operation'] = 'read'
# os.environ['service_type'] = 'ec2_zombie_resource_service'
# os.environ['service_type'] = 'iam_zombie_resource_service'
# os.environ['service_type'] = 's3_zombie_resource_service'
# os.environ['resource'] = 'zombie_cluster_elastic_ip'
# os.environ['resource'] = 'zombie_cluster_nat_gateway'
# os.environ['cluster_tag'] = ''
# os.environ['cluster_tag'] = ''
# os.environ['policy_output'] = 's3://bucket_name/logs'
# os.environ['policy_output'] = os.path.dirname(os.path.realpath(__file__))
# os.environ['policy'] = 'ebs_unattached'
# os.environ['resource_name'] = 'ocp-test'
# os.environ['user_tag_operation'] = 'read'
# os.environ['remove_tags'] = "['Manager', 'Project','Environment', 'Owner', 'Budget']"
# os.environ['username'] = 'athiruma'
# os.environ['cost_explorer_tags'] = "['User', 'Budget', 'Project', 'Manager', 'Owner', 'LaunchTime', 'Name', 'Email']"
# os.environ['file_name'] = 'tag_user.csv'
# os.environ['file_path'] = ''
# os.environ['mandatory_tags'] = "{'Budget': 'PERF-DEPT'}"
# os.environ['mandatory_tags'] = ''
# os.environ['policy'] = 'gitleaks'
# os.environ['git_access_token'] = ''
# os.environ['git_repo'] = 'https://github.com/redhat-performance'
# os.environ['several_repos'] = 'yes'
# os.environ['git_repo'] = 'https://github.com/redhat-performance/pulpperf'
# os.environ['git_repo'] = 'https://github.com/gitleakstest/gronit'
# os.environ['upload_data_elk'] = 'upload_data_elk'



from cloud_governance.policy.policy_operations.aws.zombie_non_cluster.run_zombie_non_cluster_policies import NonClusterZombiePolicy


class EbsUnattached(NonClusterZombiePolicy):
    """
    This class deletes the ebs unattached if more than 7 days, trigger mail if more than 3 days
    add Tag policy=skip/Not_Delete to skip deletion
    """

    def __init__(self):
        super().__init__()

    def run(self):
        """
        This method list all ebs unattached volumes and delete if it is unattached more than 7 days
        @return:
        """
        return self.__delete_ebs_unattached()

    def __delete_ebs_unattached(self):
        """
        This method list all ebs volumes and delete if it is unattached more than 7 days, trigger mail after 4 days
        @return:
        """
        volumes = self._ec2_client.describe_volumes(Filters=[{'Name': 'status', 'Values': ['available']}])['Volumes']
        unattached_volumes_data = []
        for volume in volumes:
            if not self._check_cluster_tag(tags=volume.get('Tags')) or self._get_policy_value(tags=volume.get('Tags')) not in ('NOTDELETE', 'SKIP'):
                volume_id = volume.get('VolumeId')
                launch_days = self._calculate_days(create_date=volume.get('CreateTime'))
                if launch_days >= self.DAYS_TO_DELETE_RESOURCE:
                    last_detached_time = self._cloudtrail.get_last_time_accessed(resource_id=volume_id,
                                                                                 event_name='DetachVolume',
                                                                                 start_time=self._start_date,
                                                                                 end_time=self._end_date,
                                                                                 optional_event_name=['CreateVolume',
                                                                                                      'CreateTags'])
                    if not last_detached_time:
                        last_detached_time = self._start_date
                    if last_detached_time:
                        last_detached_days = self._calculate_days(create_date=last_detached_time)
                        ebs_cost = self.resource_pricing.get_ebs_cost(volume_type=volume.get('VolumeType'), volume_size=volume.get('Size'), hours=(self.DAILY_HOURS * last_detached_days))
                        delta_cost = 0
                        if last_detached_days == self.DAYS_TO_NOTIFY_ADMINS:
                            delta_cost = self.resource_pricing.get_ebs_cost(volume_type=volume.get('VolumeType'), volume_size=volume.get('Size'), hours=(self.DAILY_HOURS * (self.DAYS_TO_DELETE_RESOURCE - self.DAYS_TO_NOTIFY_ADMINS)))
                        else:
                            if last_detached_days == self.DAYS_TO_DELETE_RESOURCE:
                                delta_cost = self.resource_pricing.get_ebs_cost(volume_type=volume.get('VolumeType'), volume_size=volume.get('Size'), hours=(self.DAILY_HOURS * (self.DAYS_TO_DELETE_RESOURCE - self.DAYS_TO_NOTIFY_ADMINS)))
                        unattached_volumes = self._check_resource_and_delete(resource_name='EBS Volume',
                                                                             resource_id='VolumeId',
                                                                             resource_type='CreateVolume',
                                                                             resource=volume,
                                                                             empty_days=last_detached_days,
                                                                             days_to_delete_resource=self.DAYS_TO_DELETE_RESOURCE,
                                                                             extra_purse=ebs_cost, delta_cost=delta_cost)
                        if unattached_volumes:
                            unattached_volumes_data.append({'ResourceId': volume.get('VolumeId'),
                                                            'Name': self._get_tag_name_from_tags(tags=volume.get('Tags'), tag_name='Name'),
                                                            'User': self._get_tag_name_from_tags(tags=volume.get('Tags'), tag_name='User'),
                                                            'Days': str(last_detached_days),
                                                            'Skip': self._get_tag_name_from_tags(tags=volume.get('Tags'), tag_name='Policy')
                                                            })
        return unattached_volumes_data



from cloud_governance.cloud_resource_orchestration.clouds.aws.ec2.run_cro import RunCRO
from cloud_governance.common.jira.jira import logger
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp
from cloud_governance.main.environment_variables import environment_variables


class CloudMonitor:
    """
    This class run CRO Monitoring
    """

    AWS = "AWS"
    GCP = "GCP"
    AZURE = "AZURE"
    IBM = "IBM"

    def __init__(self):
        self.__environment_variables_dict = environment_variables.environment_variables_dict
        self.__region = self.__environment_variables_dict.get('AWS_DEFAULT_REGION', '')
        self.__cloud_name = self.__environment_variables_dict.get('PUBLIC_CLOUD_NAME')
        self.__monitor = self.__environment_variables_dict.get('MONITOR')
        self.__account = self.__environment_variables_dict.get('account')
        self.__run_cro = RunCRO()

    @logger_time_stamp
    def aws_cloud_monitor(self):
        """
        This method ture if the cloud name is
        """
        self.__run_cro.run()

    @logger_time_stamp
    def run_cloud_monitor(self):
        """
        This method run the public cloud monitor
        """

        if self.__cloud_name.upper() == self.AWS:
            logger.info(f'CLOUD_RESOURCE_ORCHESTRATION = True, PublicCloudName = {self.__cloud_name}, Account = {self.__account}')
            self.aws_cloud_monitor()

    def run(self):
        """
        This method monitoring the cloud resources
        """
        self.run_cloud_monitor()
